'From Cuis 5.0 [latest update: #4392] on 14 October 2020 at 4:56:17 pm'!

!Object methodsFor: 'environment support' stamp: 'KLG 10/8/2020 16:55:42'!
environmentImplementationsIfPresentDo: aBlock
	"This method will execute aBlock with the single instance of the environment implementations directory.

     Do nothing" ! !
    
!classDefinition: #HierarchyBrowser category: #'Tools-Browser'!
Browser subclass: #HierarchyBrowser
	instanceVariableNames: 'classList centralClass classNamesList '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!
!classDefinition: #ProtocolBrowser category: #'Tools-Browser'!
MessageSet subclass: #ProtocolBrowser
	instanceVariableNames: 'baseClass selectiveClassList selectiveClassListIndex selectedName exclude selectiveClassNameList '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!Browser methodsFor: 'class list' stamp: 'KLG 10/12/2020 17:19:10'!
classNamesListForSelecting
	"Answer the class names list for selecting.
	
	Needed because we want the list of environment aware symbols
	to select from it."

	^ self listClassesHierarchically
		ifTrue: [self hierarchicalClassNamesList]
		ifFalse: [self defaultClassList].! !

!Browser methodsFor: 'class list' stamp: 'KLG 10/12/2020 17:28:51'!
flattenHierarchyTree: classHierarchy on: aCollection

	"Recursively add to col the names in classHierarchy NOT indenting to show NOTHING."

	classHierarchy do: [:assoc |
		| class children |
		class := assoc key.
		aCollection add: class name.
		children := assoc value.
		self flattenHierarchyTree: children			on: aCollection ].
	^ aCollection! !

!Browser methodsFor: 'class list' stamp: 'KLG 10/12/2020 17:29:39'!
hierarchicalClassNamesList

	"classNames are an arbitrary collection of classNames of the system.
	Reorder those class names so that they are sorted and NOT indended by inheritance.
	
	We need the orginal symbols to get the selectedClassName from."

	| classes |

	"Creating the hierarchy is *really slow* for the full class list. Skip it for now."
	selectedSystemCategory = SystemOrganizer allCategory ifTrue: [^ self defaultClassList].		
	classes := self defaultClassList collect: [:sym | Smalltalk classNamed: sym].
	^ self
		flattenHierarchyTree: (self createHierarchyTreeOf: classes)
		on: OrderedCollection new! !

!Browser methodsFor: 'class list' stamp: 'KLG 12/7/2020 22:14:28'!
selectedClassName
	"Answer the name of the current class. Answer nil if no selection exists."

	^selectedClassName ifNotNil: [ 
		"I send #defaultClassList and no #classList because when showing classes hierarchically we should remove spaces to see
		if class name is in the list and that consumes more time - Hernan"
		(self classNamesListForSelecting includes: selectedClassName) ifTrue: [ selectedClassName ]]! !

!HierarchyBrowser methodsFor: 'class list' stamp: 'KLG 10/12/2020 20:25:01'!
classNamesListForSelecting
	"Answer the class names list for selecting.
	
	Needed because we want the list of environment aware symbols
	to select from it."
	
	classNamesList _ classNamesList select: [ :each |
		Smalltalk includesKey: each ].
	^ classNamesList! !


!ClassDescription methodsFor: 'printing' stamp: 'KLG 10/7/2020 21:06:21'!
denotingExpression
	"Answer a string that evaluates to this class/metaclass.
	
	This method is redefined in class."

	^ self name! !

!ClassDescription methodsFor: 'printing' stamp: 'KLG 10/12/2020 18:15:51'!
safeDenotingExpression
	"Answer a string that evaluates to this class/metaclass, 
	even if embedded in a keyword expression
	
	This method is redefined in class."

	^ self denotingExpression! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'KLG 10/21/2020 13:55:00'!
definitionPreambleWithoutStamp

	^'classDefinition: ', self safeDenotingExpression, ' category: ', self category printString! !

!ClassDescription methodsFor: 'printing' stamp: 'KLG 10/21/2020 17:01:24'!
denotingLiteral
	"Answer a string that contains a literal that denotes to this class/metaclass.
	
	This method is redefined in class."

	^ self name printString! !

!ClassDescription methodsFor: 'printing' stamp: 'KLG 10/21/2020 17:02:19'!
safeDenotingLiteral
	"Answer a string that contains a literal that denotes to this class/metaclass.
	Do it in a way, that is a safe pirmary expression.
	
	This method is redefined in class."

	^ self denotingLiteral ! !

!ClassDescription methodsFor: 'printing' stamp: 'KLG 1/26/2021 20:20:26'!
sharedPoolsString
	"Answer a string of my shared pool names separated by spaces."

	^String streamContents: [ :stream |
		self sharedPools 
			do: [ :each |
				stream nextPutAll: (Smalltalk
					keyAtIdentityValue: each 
					ifAbsent: [ each inheritsFrom: SharedPool :: ifFalse: 'private' ifTrue: each name ]) ]
			separatedBy: [ stream space ] ]! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'KLG 10/21/2020 17:07:39'!
definitionPreambleWithoutStamp

	^'classDefinition: ', self safeDenotingLiteral, ' category: ', self category printString! !

    
    
!Class methodsFor: 'printing' stamp: 'KLG 10/7/2020 21:12:32'!
denotingExpression
	"Answer a string that evaluates to this class/metaclass.
	
	Divert to the metaclass, because the metaclass may know
	how the class can be navigated to."
	
	^ self class denotingExpressionForClass! !

!Class methodsFor: 'printing' stamp: 'KLG 10/12/2020 18:16:04'!
safeDenotingExpression
	"Answer a string that evaluates to this class/metaclass.
	
	Divert to the metaclass, because the metaclass may know
	how the class can be navigated to."
	
	^ self class safeDenotingExpressionForClass! !


!Metaclass methodsFor: 'printing' stamp: 'KLG 10/7/2020 21:54:35'!
denotingExpression
	"Answer a String that is the name of the receiver, either 'Metaclass' or 
	the name of the receiver's class followed by ' class'.
	
	Keep in mind that 'thisClass denotingExpression' might need
	parenthesis in subclasses."

	^ thisClass
		ifNil: [ 'a Metaclass' ]
		ifNotNil: [ thisClass denotingExpression, ' class' ]! !

!Class methodsFor: 'printing' stamp: 'KLG 10/21/2020 17:05:59'!
denotingLiteral
	"Answer a string that evaluates to this class/metaclass.
	
	Divert to the metaclass, because the metaclass may know
	how the class can be navigated to."
	
	^ self class denotingLiteralForClass! !

!Class methodsFor: 'printing' stamp: 'KLG 10/21/2020 17:06:27'!
safeDenotingLiteral
	"Answer a string that evaluates to this class/metaclass.
	
	Divert to the metaclass, because the metaclass may know
	how the class can be navigated to."
	
	^ self class safeDenotingLiteralForClass! !


    
!Metaclass methodsFor: 'printing' stamp: 'KLG 10/7/2020 21:10:53'!
denotingExpressionForClass
	"Answer a string that evaluates to this class.
	
	Can be redefined in subclasses to prorvide support for
	navigating to a class in an environment."

	^ thisClass name! !

!Metaclass methodsFor: 'printing' stamp: 'KLG 10/12/2020 18:15:12'!
safeDenotingExpressionForClass
	"Answer a string that evaluates to this class.
	
	Answer an expression, that can savly used in every other expression,
	that is add parentheses if necessary.
	
	Can be redefined in subclasses to prorvide support for
	navigating to a class in an environment."

	^ self denotingExpressionForClass! !

!Metaclass methodsFor: 'printing' stamp: 'KLG 10/21/2020 17:04:36'!
denotingLiteralForClass
	"Answer a string that  contains a literal that denotes this class.
	
	Can be redefined in subclasses to provide support for
	navigating to a class in an environment."

	^ thisClass name printString! !

!Metaclass methodsFor: 'printing' stamp: 'KLG 10/21/2020 17:05:11'!
safeDenotingLiteralForClass
	"Answer a string that  contains a literal that denotes this class.
	Answer a safe primary expression.
	
	Can be redefined in subclasses to provide support for
	navigating to a class in an environment."

	^ self denotingLiteralForClass! !


    
!BasicClassOrganizer methodsFor: 'fileIn/Out' stamp: 'KLG 10/21/2020 12:34:41'!
putCommentOnFile: aFileStream numbered: sourceIndex moveSource: moveSource forClass: aClass
	"Store the comment about the class onto file, aFileStream."
	| header |
	self hasClassComment ifTrue: [
		aFileStream newLine; nextPut: $!!.
		header _ String streamContents: [:strm | 
				strm nextPutAll: aClass safeDenotingExpression;
				nextPutAll: ' commentStamp: '.
				commentStamp ifNil: [commentStamp _ '<historical>'].
				commentStamp storeOn: strm.
				strm nextPutAll: ' prior: '; nextPutAll: '0'].
		aFileStream nextChunkPut: header.
		aClass organization fileOutCommentOn: aFileStream
				moveSource: moveSource toFile: sourceIndex.
		aFileStream newLine]! !

    

!Browser methodsFor: 'class list' stamp: 'KLG 10/12/2020 18:07:35'!
classListIndex: anInteger 
	"Set anInteger to be the index of the current class selection."

	| className recent canSelectClass classList |

	classList _ self classNamesListForSelecting.
	canSelectClass _ classList isInBounds: anInteger.
	selectedClassName _ canSelectClass ifTrue: [	| newClassName |
		newClassName := classList at: anInteger ifAbsent: [ nil ].
		newClassName ].
	self setClassOrganizer.
	selectedMessage _ nil.
	
	self classCommentIndicated ifFalse: [
		self editSelection: (canSelectClass
			ifTrue: [#editClass]
			ifFalse: [ metaClassIndicated | selectedSystemCategory isNil
				ifTrue: [#none]
				ifFalse: [#newClass]])].

	self selectedClass ifNotNil: [
		recent _ self class recentClasses.
		className _ self selectedClass name.
		(recent includes: className) ifTrue: [recent remove: className].
		recent addFirst: className.
		recent size > 16 ifTrue: [recent removeLast]].

	"Clear selectedMessageCategory if there is no match in the new list of categories"
	(self messageCategoryList indexOf: selectedMessageCategory) = 0 ifTrue: [
		selectedMessageCategory _ nil].
		
	"We have to refresh the class list if somebody wants to select a class that does not exist anymore - Hernan"
	anInteger > classList size ifTrue: [ self changed: #classList ].
	self changed: #classSelectionChanged.
	self changed: #classCommentText.
	self changed: #classListIndex.	"update my selection"
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #relabel.
	self acceptedContentsChanged! !


!Browser methodsFor: 'class list' stamp: 'KLG 10/15/2020 13:05:22'!
classListIndexOf: aClassNameToFind

	"Answer the index of the aClassName selection."

	aClassNameToFind ifNil: [ ^0 ].
	^self classNamesListForSelecting findFirst: [ :showingClassName |
		"Works regardless of currently showing hierarchically or alphabetically."
		showingClassName == aClassNameToFind  ]! !

    
!Browser methodsFor: 'system category functions' stamp: 'KLG 10/12/2020 17:35:29'!
categorizeUnderCategoryAt: aSystemCategoryIndex class: aClassName 

	systemOrganizer classify: aClassName under: (self systemCategoryList at: aSystemCategoryIndex).
	self changed: #classList! !
    
!Browser methodsFor: 'system category functions' stamp: 'KLG 10/12/2020 17:35:54'!
categorizeUnderNewCategoryClass: aClassName

	| newCategory |
	newCategory _ self newSystemCategoryNameIfNone: [ ^self ].
	
	systemOrganizer
		addCategory: newCategory;
		classify: aClassName under: newCategory.
		
	self changed: #systemCategoryRoots.
	self changed: #classList.! !

!Browser methodsFor: 'system category functions' stamp: 'KLG 5/5/2022 14:00:51'!
moveAllToOtherSystemCategory
	"If a class category is selected, prompt user for category to move to,
	create a Confirmer so the user can verify that all the classes in current category
 	should be moved to the selected category."
	| newSystemCategory |
	selectedSystemCategory ifNil: [ ^ self ].
	newSystemCategory _ Smalltalk systemCategoryFromUserWithPrompt: 'Move classes to System Category...'.
	(newSystemCategory notNil and: [
		self classNamesListForSelecting notEmpty and: [ self confirm: 'Are you sure you want to
move classes from ' , selectedSystemCategory , ' 
to ' , newSystemCategory , '?' ]]) ifTrue: [
		"Safer this way (#classList will be a collection of strings with spaces and who knows what in the future.  So let's just get the classes we need directly)"
		(SystemOrganization classesAt: selectedSystemCategory) do: [ :eaClass |
			eaClass category: newSystemCategory ].
		self changed: #systemCategoryRoots ].! !


!Browser methodsFor: 'system category functions' stamp: 'KLG 5/5/2022 14:12:35'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."

	selectedSystemCategory ifNil: [^ self].
	(self classNamesListForSelecting isEmpty
		or: [self confirm: 'Are you sure you want to
remove this system category 
and all its classes?'])
		ifTrue: [
			systemOrganizer removeSystemCategory: selectedSystemCategory.
			self changed: #systemCategoryRoots ]! !


!Browser methodsFor: 'system category functions' stamp: 'KLG 2/19/2022 20:13:21'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."

	selectedSystemCategory ifNil: [^ self].
	(self classNamesListForSelecting size = 0
		or: [self confirm: 'Are you sure you want to
remove this system category 
and all its classes?'])
		ifTrue: [
			systemOrganizer removeSystemCategory: selectedSystemCategory.
			self changed: #systemCategoryRoots ]! !


!HierarchyBrowser methodsFor: 'initialization' stamp: 'KLG 10/12/2020 20:20:27'!
classListIndex: newIndex

	"Cause system organization to reflect appropriate category"

	| newClassName ind i |
	(classNamesList isInBounds: newIndex) ifTrue: [ 
		newClassName _ classNamesList at: newIndex.
		i _ systemOrganizer numberOfCategoryOfElement: newClassName.
		selectedSystemCategory _ i = 0 ifFalse: [ self systemCategoryList at: i]].
	ind _ super classListIndex: newIndex.
	self changed: #systemCategorySingleton.
	^ ind! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'KLG 10/12/2020 20:28:12'!
initAlphabeticListing
	
	self systemOrganizer: SystemOrganization.
	metaClassIndicated _ false.
	classList _ Smalltalk classNames.
	classNamesList _ Smalltalk classNames.! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'KLG 10/12/2020 20:09:27'!
initHierarchyForClass: aClassOrMetaClass
	| tab stab index nonMetaClass |
	centralClass _ aClassOrMetaClass.
	nonMetaClass _ aClassOrMetaClass theNonMetaClass.
	self systemOrganizer: SystemOrganization.
	metaClassIndicated _ aClassOrMetaClass isMeta.
	classList _ OrderedCollection new.
	classNamesList _ OrderedCollection new.
	tab _ ''.
	nonMetaClass allSuperclasses reverseDo: [ :aClass | 
		classList add: tab , aClass name.
		tab _ tab , '  '.
		classNamesList add: aClass name ].
	index _ classList size + 1.
	nonMetaClass allSubclassesWithLevelDo: [ :aClass :level |
		stab _ ''.  1 to: level do: [ :i |
			stab _ stab , '  '].
			classList add: tab , stab , aClass name.
			classNamesList add: aClass name ]
	 	startingLevel: 0.
	self classListIndex: index! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'KLG 10/12/2020 20:35:01'!
potentialClassNames
	
	"Answer the names of all the classes that could be viewed in this browser"
	
	^ self classNamesListForSelecting! !

!HierarchyBrowser methodsFor: 'class list' stamp: 'KLG 10/14/2020 11:47:20'!
classList

	| newClassList |
	newClassList _ OrderedCollection new: classList size.
	classList with: classNamesList do: [ :displayName :nameSymbol |
		Smalltalk includesKey: nameSymbol :: ifTrue: [
			newClassList add: displayName ] ].
	classList _ newClassList.
	^ classList! !


!ProtocolBrowser methodsFor: 'private' stamp: 'KLG 10/14/2020 16:51:45'!
hierarchyForClass: aClass
	"Set the class hierarchy for the list pane"
	| tab |

	selectiveClassList _ OrderedCollection new.
	selectiveClassNameList _ OrderedCollection new.
	tab _ ''.
	aClass withAllSuperclasses reverse do: [:ea |
		selectiveClassList add: tab , ea name.
		selectiveClassNameList add: ea name.
		tab _ tab , '  '].
	self classListIndex: 0! !

!ProtocolBrowser methodsFor: 'private' stamp: 'KLG 10/14/2020 16:52:59'!
protocolFor: anIndex
	"Change the listed protocol"

	exclude _ OrderedCollection new.
	anIndex > 0
		ifTrue: [
			selectedName _ selectiveClassNameList at: anIndex.
			(1 to: anIndex - 1) do: [ :ix |
				exclude addLast: (selectiveClassNameList at: ix) ]]
		ifFalse: [
			selectedName _ nil.
			].
	self on: baseClass.
	self changed: #relabel! !

!ProtocolBrowser methodsFor: 'class list' stamp: 'KLG 10/14/2020 16:53:59'!
selectedClass
	"Answer the class that is currently selected. 
	Answer base if no selection exists."
	| className |
	className := (self selectiveClassListIndex = 0) 
					ifTrue:  [selectiveClassNameList last]
					ifFalse:  [selectiveClassNameList at: self selectiveClassListIndex].
	^ Smalltalk at: className! !


!ClassDescription methodsFor: 'fileIn/Out' stamp: 'KLG 10/12/2020 18:15:51'!
definition
	"Answer a String that defines the receiver."

	^String streamContents: [ :strm |
		strm
			nextPutAll: (
				superclass 
					ifNotNil: [ superclass safeDenotingExpression ] 
					ifNil: [ 'ProtoObject' ]);
			nextPutAll: self kindOfSubclass;
			store: self name.
		strm
			newLine;
			tab;
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString.
		strm
			newLine;
			tab;
			nextPutAll: 'classVariableNames: ';
			store: self classVariablesString.
		strm
			newLine;
			tab;
			nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString.
		strm
			newLine;
			tab;
			nextPutAll: 'category: ';
			store: self category asString.

		"Add the environment or other information to the source code."
		self printAdditionalDefinitionOn: strm.

		superclass ifNil: [ 
			strm nextPutAll: '.'; newLine.
			strm nextPutAll: self name.
			strm space; nextPutAll: 'superclass: nil' ]]! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'KLG 10/7/2020 20:53:07'!
printAdditionalDefinitionOn: aStream
	"Print additional defintion information on aStream.
	
	Sends of this message are diverted to the metaclass in Class
	to support additional definition information, such as environments.
	Metaclass inherits this no-op implementation."! !


!ClassDescription methodsFor: 'fileIn/Out' stamp: 'KLG 5/28/2022 11:23:53'!
printMethodChunk: selector withPreamble: doPreamble on: outStream moveSource: moveSource toFile: fileIndex
	"Copy the source code for the method associated with selector onto the fileStream.  If moveSource true, then also set the source code pointer of the method."
	| preamble compiledMethod oldPos newPos sourceFile endPos |
	doPreamble 
		ifTrue: [preamble _ self safeDenotingExpression, ' methodsFor: ' ,
					(self organization categoryOfElement: selector) asPlainString printString]
		ifFalse: [preamble _ ''].
	compiledMethod _ self methodDict at: selector ifAbsent: [
		outStream nextPutAll: selector; newLine.
		outStream tab; nextPutAll: '** ERROR  -  THIS METHOD IS MISSING ** '; newLine; newLine.
		outStream nextPutAll: '  '.
		^ outStream].

	((compiledMethod fileIndex = 0
		or: [(SourceFiles at: compiledMethod fileIndex) == nil])
		or: [(oldPos _ compiledMethod filePosition) = 0])
	ifTrue: [
		"The source code is not accessible.  We must decompile..."
		preamble size > 0 ifTrue: [ outStream newLine; nextPut: $!!; nextChunkPut: preamble; newLine].
		outStream nextChunkPut: compiledMethod decompileString]
	ifFalse: [
		sourceFile _ SourceFiles at: compiledMethod fileIndex.
		preamble size > 0
			ifTrue: [   "Copy the preamble"
				outStream copyPreamble: preamble oldPreamble: compiledMethod getPreamble ].
		sourceFile position: oldPos.
		"Copy the method chunk"
		fileIndex = 0 ifFalse: [
			outStream padTo: SourceFiles pointerScaleForWriting put: $  ].
		newPos _ outStream position.
		outStream copyMethodChunkFrom: sourceFile.
		moveSource ifTrue: [    "Set the new method source pointer"
			endPos _ outStream position.
			compiledMethod checkOKToAdd: endPos - newPos at: newPos in: compiledMethod fileIndex.
			compiledMethod setSourcePosition: newPos inFile: fileIndex]].
	preamble notEmpty ifTrue: [ outStream nextChunkPut: ' ' ].
	^ outStream newLine! !    
    
!ClassDescription methodsFor: 'printing' stamp: 'KLG 10/7/2020 21:07:35'!
printOn: aStream 

	aStream nextPutAll: self denotingExpression! !


!Class methodsFor: 'compiling' stamp: 'KLG 10/7/2020 20:43:50'!
nonLocalBindingOf: aVarName
	"Answer the non local binding of aVarName.
	
	Answer nil if no binding is available."

	"#forNonLocalBindingEnvironmentsDo: can be redefined in 
	subclasses of Metaclass to provide support for environments.
	In Metaclass it defaults to Smalltalk."
	self class forNonLocalBindingEnvironmentsDo: [ :environment |
		environment bindingOf: aVarName :: ifNotNil: [ :binding | ^ binding ] ].
	^ nil ! !


!Metaclass methodsFor: 'compiling' stamp: 'KLG 10/7/2020 20:55:33'!
forNonLocalBindingEnvironmentsDo: aBlock
	"Evaluate aBlock for all non local environments.

	Maybe redefined in meta classes, that provide support/for
	environments/namespaces.

	Ordinary metaclasses only provide Smalltalk as non-local namespace."

	aBlock value: Smalltalk! !

   
!Class methodsFor: 'compiling' stamp: 'KLG 10/6/2020 14:03:40'!
binding

	^ self nonLocalBindingOf: name :: ifNil: [ nil -> self ]! !

!Class methodsFor: 'compiling' stamp: 'KLG 10/7/2020 20:41:11'!
bindingOf: varName
	"Answer the binding of some variable resolved in the scope of the receiver"
	| aSymbol |
	aSymbol _ varName asSymbol.

	"First look in receiver, and up the hierarchy. Inherited variables take precedence over globals."
	(self localBindingOf: aSymbol) ifNotNil: [ :binding | ^binding ].

	"Next look in globals."
	(self nonLocalBindingOf: aSymbol) ifNotNil: [ :binding | ^binding ].

	"Fail at the end."
	^nil! !

!Class methodsFor: 'fileIn/Out' stamp: 'KLG 10/7/2020 20:45:38'!
printAdditionalDefinitionOn: aStream
	"Print additional defintion information on aStream.
	
	Divert to the metaclass, which may print, if subclassed,
	environment information or other stuff.
	
	The implementation inherited by Metaclass form ClassDescription
	does nothing."
	
	self class printAdditionalDefinitionOn: aStream! !


!Class class methodsFor: 'instance creation' stamp: 'KLG 10/12/2020 19:01:13'!
templateForSubclassOf: priorClassName category: systemCategoryName 
	"Answer an expression that can be edited and evaluated in order to define a new class, given that the class previously looked at was as given"

	| superClassString |
	superClassString _ priorClassName asString.
	Scanner isLiteralSymbol: superClassString :: ifFalse: [
		superClassString _ '(', superClassString, ')' ].
	^ superClassString, ' subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''' , systemCategoryName asPlainString , ''''! !


!Class methodsFor: 'initialization' stamp: 'KLG 1/26/2021 20:20:20'!
sharedPoolAt: aNameSymbol ifAbsent: aBlock
	"Answer the shared pool at aNameSymbol.
	
	Ansewr the value of a block if there is none."

	^ self nonLocalBindingOf: aNameSymbol :: ifNil: aBlock ifNotNil: [ :binding | binding value ]! !

!Class methodsFor: 'initialization' stamp: 'KLG 1/26/2021 19:54:03'!
sharing: poolString 
	"Set up sharedPools. Answer whether recompilation is advisable."
	| oldPools |
	oldPools _ self sharedPools.
	sharedPools _ OrderedCollection new.
	(Scanner new scanFieldNames: poolString) do: 
		[:poolName | 
		sharedPools add: (self sharedPoolAt: poolName asSymbol ifAbsent:[
			(PoolDefinitionNotification signalNamed: poolName)
				ifTrue:[Smalltalk at: poolName asSymbol put: Dictionary new]
				ifFalse:[^self error: poolName,' does not exist']])].
	sharedPools isEmpty ifTrue: [sharedPools _ nil].
	^oldPools anySatisfy: [ :pool |
		self sharedPools noneSatisfy: [ :p | p == pool ]]! !


!ClassBuilder methodsFor: 'private' stamp: 'KLG 10/7/2020 20:48:10'!
privateNewSubclassOf: newSuper
	"Create a new meta and non-meta subclass of newSuper"
	"WARNING: This method does not preserve the superclass/subclass invariant!!"

	| newSuperMeta newMeta |
	newSuperMeta _ newSuper ifNil:[ Class ] ifNotNil:[ newSuper class ].
	newMeta _ self metaclassClass new.
	self configureNewMetaclass: newMeta.
	newMeta 
		superclass: newSuperMeta 
		methodDictionary: MethodDictionary new 
		format: newSuperMeta format.
	^ newMeta new
! !

!ClassBuilder methodsFor: 'private' stamp: 'KLG 10/7/2020 20:48:58'!
privateNewSubclassOf: newSuper from: oldClass
	"Create a new meta and non-meta subclass of newSuper using oldClass as template"
	"WARNING: This method does not preserve the superclass/subclass invariant!!"

	| newSuperMeta oldMeta newMeta |
	oldClass ifNil:[ ^ self privateNewSubclassOf: newSuper ].
	newSuperMeta _ newSuper ifNil:[ Class ] ifNotNil:[ newSuper class ].
	oldMeta _ oldClass class.
	newMeta _ self newMetaclassFromOldMetaclass: oldMeta.
	self configureNewMetaclass: newMeta.
	newMeta 
		superclass: newSuperMeta
		methodDictionary: MethodDictionary new
		format: (self computeFormat: oldMeta typeOfClass 
					instSize: oldMeta instVarNames size 
					forSuper: newSuperMeta
					ccIndex: 0);
		setInstVarNames: oldMeta instVarNames;
		organization: oldMeta organization.
	"Recompile the meta class"
	oldMeta hasMethods 
		ifTrue:[ newMeta compileAllFrom: oldMeta ].
	"Record the meta class change"
	self recordClass: oldMeta replacedBy: newMeta.
	"And create a new instance"
	^ newMeta adoptInstance: oldClass from: oldMeta! !

!ClassBuilder methodsFor: 'instance creation' stamp: 'KLG 10/7/2020 20:47:10'!
metaclassClass
	"Answer the proper meta class for me."

	^ self class metaclassClass! !

!ClassBuilder methodsFor: 'subclass hooks' stamp: 'KLG 10/7/2020 20:46:26'!
configureNewMetaclass: aMetaclass
	"Configure a newly created  metaclass.
	
	The aMetaclass maybe the result of a shallow copy, so take care.
	
	Subclass me if you need special configuration."! !

!ClassBuilder methodsFor: 'subclass hooks' stamp: 'KLG 10/7/2020 20:47:26'!
newMetaclassFromOldMetaclass: aMetaclass
	"Create a new meta class instance from the old one.

 	Override me if you have a custom class builder with custom
	metclasses that need some tweaking."

	^ aMetaclass shallowCopy! !

!ClassBuilder methodsFor: 'subclass hooks' stamp: 'KLG 11/16/2020 13:42:23'!
classFromClassName: aClassName
	"Answer the class denoted by a ClassName."

	^ Smalltalk at: aClassName ifAbsent: [ nil ]! !

!ClassBuilder methodsFor: 'class definition' stamp: 'KLG 11/16/2020 13:42:58'!
name: className subclassOf: newSuper type: type instanceVariableNames: instVarString classVariableNames: classVarString poolDictionaries: poolString category: category unsafe: unsafe
	"Define a new class.
	If unsafe is true do not run any validation checks.
	This facility is provided to implement important system changes."
	
	| oldClass newClass organization instVars classVars force needNew oldCategory copyOfOldClass newCategory |
	
	instVars _ Scanner new scanFieldNames: instVarString.
	classVars _ (Scanner new scanFieldNames: classVarString) collect: [:x | x asSymbol].

	"Validate the proposed name"
	unsafe ifFalse:[(self validateClassName: className) ifFalse:[^nil]].
	oldClass _ self classFromClassName: className.
	oldClass isBehavior 
		ifFalse:[oldClass _ nil]. "Already checked in #validateClassName:"
	copyOfOldClass _ oldClass copy.

	unsafe ifFalse:[
		"Run validation checks so we know that we have a good chance for recompilation"
		(self validateSuperclass: newSuper forSubclass: oldClass) ifFalse:[^nil].
		(self validateInstvars: instVars from: oldClass forSuper: newSuper) ifFalse:[^nil].
		(self validateClassvars: classVars from: oldClass forSuper: newSuper) ifFalse:[^nil].
		(self validateSubclassFormat: type from: oldClass forSuper: newSuper extra: instVars size) ifFalse:[^nil]].

	"See if we need a new subclass"
	needNew _ self needsSubclassOf: newSuper type: type instanceVariables: instVars from: oldClass.
	needNew ifNil: [^nil]. "some error"

	(needNew and:[unsafe not]) ifTrue:[
		"Make sure we don't redefine any dangerous classes"
		(self tooDangerousClasses includes: oldClass name) ifTrue:[
			self error: oldClass name, ' cannot be changed'.
		].
		"Check if the receiver should not be redefined"
		(oldClass notNil and:[oldClass shouldNotBeRedefined]) ifTrue:[
			self notify: oldClass name asText allBold, 
						' should not be redefined!! \Proceed to store over it.' withNewLines]].

	needNew ifTrue:[
		"Create the new class"
		newClass _ self 
			newSubclassOf: newSuper 
			type: type 
			instanceVariables: instVars
			from: oldClass.
		newClass ifNil: [ ^nil]. "Some error"
		newClass setName: className.
	] ifFalse:[
		"Reuse the old class"
		newClass _ oldClass.
	].

	"Install the class variables and pool dictionaries... "
	force _ (newClass declare: classVarString) | (newClass sharing: poolString).

	"... classify ..."
	newCategory _ category withBlanksTrimmed.
	organization _ Smalltalk organization.
	oldClass ifNotNil: [oldCategory := (organization categoryOfElement: oldClass name) asSymbol].
	organization classify: newClass name under: newCategory.

	"... recompile ..."
	newClass _ self recompile: force from: oldClass to: newClass mutate: false.

	"... export if not yet done ..."
	(Smalltalk at: newClass name ifAbsent: nil) == newClass ifFalse:[
		[Smalltalk at: newClass name put: newClass]
			on: AttemptToWriteReadOnlyGlobal do:[:ex| ex resume: true].
		Smalltalk flushClassNameCache.
	].

	self doneCompiling: newClass.
	
	"... notify interested clients ..."
	oldClass ifNil: [
		SystemChangeNotifier uniqueInstance classAdded: newClass inCategory: newCategory.
		^ newClass].
	SystemChangeNotifier uniqueInstance classDefinitionChangedFrom: copyOfOldClass to: newClass.
	newCategory ~= oldCategory 
		ifTrue: [SystemChangeNotifier uniqueInstance classRecategorized: newClass from: oldCategory to: newCategory ].
	^newClass! !


!ClassBuilder class methodsFor: 'instance creation' stamp: 'KLG 10/7/2020 20:49:34'!
metaclassClass
	"Answer the proper meta class for me.
	
	Subclass for specialized class builders."

	^ Metaclass! !


!SystemDictionary methodsFor: 'class names' stamp: 'KLG 12/9/2020 14:46:34'!
classNamed: className 
	"className is either a class name or a class name followed by ' class'.
	Answer the class or metaclass it names.
	Answer nil if no class by that name."
	"
	Smalltalk classNamed: #Point
	Smalltalk classNamed: 'Point'
	Smalltalk classNamed: 'Point class'
	Smalltalk classNamed: 'BogusClassName'
	Smalltalk classNamed: 'BogusClassName class'

	Smalltalk classNamed: #Display
	Smalltalk classNamed: 'Display'
	Smalltalk classNamed: 'Display class'
	"

	self environmentImplementationsIfPresentDo: [ :implementations |
		className doWithEnvironmentIfAware: [ :environment |
			^ environment classNamed: className ].
		"This is not only overkill, it's wrong in the first place!!"
		"O: implementations classNamed: className :: ifNotNil: [ :classFound |
			^ classFound ]" ].
	Smalltalk at: className asSymbol ifPresent: [ :found |
		^ found isBehavior ifTrue: [ found ]].

	(className withoutSuffix: ' class') ifNotNil: [ :baseName |
		Smalltalk at: baseName asSymbol ifPresent: [ :found |
			^ found isBehavior ifTrue: [ found class ]]].

	^ nil! !

!SystemDictionary methodsFor: 'class names' stamp: 'KLG 10/7/2020 21:52:09'!
fillCaches
	"Fill cachedClassNames and cachedNonClassNames. Return an array with the calculated values."

	| classNames nonClassNames |
	classNames _ IdentitySet new: self size.
	nonClassNames _ IdentitySet new.
	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations
			fillClassNameCache: classNames
			andNonClassNameCache: nonClassNames ].
	self keysAndValuesDo: [ :key :value |
		value isInMemory ifTrue: [
			"The key == value name test below addresses two separate issues:
				1) Obsolete classes, where key = #Foo and value name = 'AnObsoleteFoo'
				2) Aliases, i.e., Smalltalk at: #OtherName put: aClass"
			(value class isMeta and: [ key == value name ])
				ifTrue: [ classNames add: key ]
				ifFalse: [ nonClassNames add: key ] ] ].
	classNames _ classNames asArray sort.
	nonClassNames _ nonClassNames asArray sort.
	cachedClassNames _ classNames.
	cachedNonClassNames _ nonClassNames.
	^{ classNames. nonClassNames }! !

!SystemDictionary methodsFor: 'class names' stamp: 'KLG 10/12/2020 10:38:09'!
hasClassNamed: aString
	"Answer whether there is a class of the given name, but don't intern aString if it's not already interned.  4/29/96 sw
	Smalltalk hasClassNamed: 'Morph'
	Smalltalk hasClassNamed: 'Display'
	Smalltalk hasClassNamed: 'xMorph'
	"

	self environmentImplementationsIfPresentDo: [ :implementations |
		"Maybe aString is actually a symbol:"
		aString doWithEnvironmentIfAware: [ :environment |
			^ environment hasClassNamed: aString ].
		implementations hasClassNamed: aString :: ifTrue: [ ^ true ] ].
	Symbol hasInterned: aString ifTrue: [ :aSymbol |
		self at: aSymbol ifPresent: [ :global | ^global class isMeta ]].
	^ false! !

!SystemDictionary methodsFor: 'class names' stamp: 'KLG 10/12/2020 10:40:11'!
prepareToRenameClass: aClass as: newName 

	self environmentImplementationsIfPresentDo: [ :implementations |
		aClass name doWithEnvironmentIfAware: [ :environment |
			^ environment prepareToRenameClass: aClass as: newName ].
		implementations prepareToRenameClass: aClass as: newName :: ifTrue: [ ^ self ] ].
	^ self prepareToRenameClass: aClass from: aClass name to: newName! !

!SystemDictionary methodsFor: 'class names' stamp: 'KLG 10/12/2020 10:42:11'!
removeClassNamed: aName
	"Invoked from fileouts:  if there is currently a class in the system named aName, then remove it.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	self environmentImplementationsIfPresentDo: [ :implementations |
		aName doWithEnvironmentIfAware: [ :environment |
			^ environment removeClassNamed: aName ].
		implementations removeClassNamed: aName :: ifTrue: [		^ self ] ].
	oldClass _ self at: aName asSymbol ifAbsent: [
		Transcript newLine; show: 'Removal of class named ', aName, ' ignored because ', aName, ' does not exist.'.
		^ self].

	oldClass removeFromSystem! !

!SystemDictionary methodsFor: 'class names' stamp: 'KLG 10/7/2020 21:52:09'!
renameClassNamed: oldName as: newName
	"Invoked from fileouts:  if there is currently a class in the system named oldName, then rename it to newName.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations renameClassNamed: oldName as: newName :: ifTrue: [
			^ self ] ].
	(oldClass _ self at: oldName asSymbol ifAbsent: nil)
		ifNil: [
			Transcript newLine; show: 'Class-rename for ', oldName, ' ignored because ', oldName, ' does not exist.'.
			^ self].

	oldClass rename: newName! !

!SystemDictionary methodsFor: 'dictionary access' stamp: 'KLG 10/12/2020 10:13:10'!
at: aKey put: anObject 
	"Override from Dictionary to check Undeclared and fix up
	references to undeclared variables."

	| association |
	"Let objects that belong to an environment, handle their installation."
	self environmentImplementationsIfPresentDo: [ :implementations |
		"We don't ask the implementations dictionary for support here,
		instead we ask the key for its environment if it is environment aware."
		aKey doWithEnvironmentIfAware: [ :environment |
			^ environment at: aKey put: anObject ] 		].
	(self includesKey: aKey) ifFalse: [
		self flushClassNameCache.

		"Update existing association if there is one."
		(Undeclared includesKey: aKey) ifTrue: [
			association _ self declare: aKey from: Undeclared.
			association value: anObject.
			^ anObject ]].

	"Update existing association if there is one."
	^super at: aKey put: anObject! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'KLG 10/7/2020 21:52:09'!
cleanOutUndeclared
	"Remove all unreferences classes."
	
	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations cleanOutUndeclared ].
	Undeclared removeUnreferencedKeys! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'KLG 10/7/2020 21:52:09'!
obsoleteClasses   "Smalltalk obsoleteClasses inspect"
	"NOTE:  Also try inspecting comments below"
	| obs c |
	obs _ OrderedCollection new.  Smalltalk garbageCollect.
	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations addObsoleteClassesTo: obs ].
	Metaclass allInstancesDo:
		[:m | c _ m soleInstance.
		(c notNil and: ['AnOb*' match: c name asPlainString])
			ifTrue: [obs add: c]].
	^ obs asArray

"Likely in a ClassDict or Pool...
(Association allInstances select: [:a | (a value isKindOf: Class) and: ['AnOb*' match: a value name]]) asArray
"
"Obsolete class refs or super pointer in last lit of a method...
| n l found |
Smalltalk browseAllSelect:
	[:m | found _ false.
	1 to: m numLiterals do:
		[:i | (((l _ m literalAt: i) isMemberOf: Association)
				and: [(l value isKindOf: Behavior)
				and: ['AnOb*' match: l value name]])
			ifTrue: [found _ true]].
	found]
"! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'KLG 10/7/2020 21:52:09'!
allBehaviorsDo: aBlock 
	"Evaluate the argument, aBlock, for each kind of Behavior in the system 
	(that is, Object and its subclasses).
	ar 7/15/1999: The code below will not enumerate any obsolete or anonymous
	behaviors for which the following should be executed:

		Smalltalk allObjectsDo:[:obj| obj isBehavior ifTrue:[aBlock value: obj]].

	but what follows is way faster than enumerating all objects."

	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations allBehaviorsDo: aBlock ].
	Smalltalk do: [ :root |
		(root isBehavior and: [root superclass isNil]) ifTrue: [	"Grab ProtoObject and any other alike"
			root withAllSubclassesDo: [ :class |
				class isMeta ifFalse: [ "The metaclasses are rooted at Class; don't include them twice."
					aBlock
						value: class;
						value: class class ]]]]! !

!SystemDictionary methodsFor: 'shrinking' stamp: 'KLG 10/12/2020 10:44:27'!
removeSelector: descriptor
	"Safely remove a selector from a class (or metaclass). If the class
	or the method doesn't exist anymore, never mind and answer nil.
	This method should be used instead of 'Class removeSelector: #method'
	to omit global class references."

	| class sel |
	self environmentImplementationsIfPresentDo: [ :environments |
		descriptor first doWithEnvironmentIfAware: [ :environment |
			^ environment removeSelector: descriptor ].
		environments removeSelector: descriptor :: ifNotNil: [  :affectedBehaviour |
			^ affectedBehaviour ] ].
	class _ Smalltalk at: descriptor first ifAbsent: [^ nil].
	(descriptor size > 2 and: [descriptor second == #class])
		ifTrue:
			[class _ class class.
			sel _ descriptor third]
		ifFalse: [sel _ descriptor second].
	^ class removeSelector: sel! !

!SystemDictionary methodsFor: 'removing' stamp: 'KLG 10/13/2020 17:36:54'!
removeKey: key ifAbsent: aBlock
	"Remove key (and its associated value) from the receiver. If key is not in
	the receiver, answer the result of evaluating aBlock. Otherwise, answer
	the value externally named by key."
	self environmentImplementationsIfPresentDo: [ :implementations |
		key doWithEnvironmentIfAware: [ :environment | | answer |
			answer _ environment removeKey: key ifAbsent: aBlock.
			self flushClassNameCache.
			^ answer ] ].
	self flushClassNameCache.
	^super removeKey: key ifAbsent: aBlock! !


!CodeFileBrowserWindow methodsFor: 'commands' stamp: 'KLG 10/12/2020 17:41:49'!
findClass
	| pattern foundClass classNames index foundCodeFile |
	self okToChange ifFalse: [^ self flash].
	self request: 'Class name?' do: [:aString|
		aString isEmpty ifFalse:
			[pattern _ aString asLowercase.
			classNames _ Set new.
			classNames addAll: model caseCodeSource classDictionary keys.
			classNames _ classNames asArray select: 
				[:n | (n asLowercase indexOfSubCollection: pattern startingAt: 1) > 0].
			classNames isEmpty ifFalse:
				[index _ classNames size = 1
					ifTrue: [1]
					ifFalse: [(PopUpMenu labelArray: classNames lines: #()) startUpMenu].
				index = 0 ifFalse:
					[foundCodeFile _ nil.
					foundClass _ nil.
					(model caseCodeSource classDictionary includesKey: (classNames at: index))
						ifTrue:
							[foundClass := model caseCodeSource classDictionary at: (classNames at: index).
							foundCodeFile := model caseCodeSource].
					foundClass ifNotNil:
						[model systemCategoryListIndex: (model systemCategoryList indexOf: foundCodeFile name asSymbol).
						model classListIndex: (model classNamesListForSelecting indexOf: foundClass name)]]]]]! !


!CodeFileBrowser methodsFor: 'class list' stamp: 'KLG 10/19/2020 21:19:20'!
classNamesListForSelecting
	"Answer the class names list for selecting.
	
	Needed because we want the list of environment aware symbols
	to select from it.
	
	In this browser class we have the right symbols in the ordinary classList"
	
	^ self classList

! !


!ChangeList methodsFor: 'scanning' stamp: 'KLG 10/19/2020 13:37:18'!
scanClassDenotation: aDenotation.
	"Scan a class denotation."
	
	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations scanClassDenotation: aDenotation :: ifNotNil: [ :scannedDenotation |
			^ scannedDenotation ] ].
	^ aDenotation! !


!CodeFile methodsFor: 'private' stamp: 'KLG 10/20/2020 15:42:09'!
scanClassDenotation: aDenotation.
	"Scan a class denotation."
	
	self environmentImplementationsIfPresentDo: [ :implementations |
		implementations scanClassDenotation: aDenotation :: ifNotNil: [ :scannedDenotation |
			^ scannedDenotation ] ].
	^ aDenotation! !

!CodeFile methodsFor: 'private' stamp: 'KLG 10/20/2020 16:57:10'!
classDefinition: string with: chgRec

	| tokens theClass |
	tokens _ Scanner new scanTokens: string.
	tokens second == #subclass: ifFalse:[ ^ doIts add: chgRec ].
	theClass _ self getClass: (tokens at: 3).
	theClass definition: string.
	classOrder add: theClass.! !

!CodeFile methodsFor: 'private' stamp: 'KLG 10/19/2020 20:58:13'!
getClass: aClassDenotation
	| pseudoClass className |
	className _ self scanClassDenotation: aClassDenotation.
	(classes includesKey: className) ifTrue:[
		^classes at: className.
	].
	pseudoClass := PseudoClass new.
	pseudoClass name: className.
	classes at: className put: pseudoClass.
	^pseudoClass.! !


!ChangeRecord methodsFor: 'access' stamp: 'KLG 12/6/2020 14:54:08'!
changeClass
	| methodClassName methodClass |
	(#(method classComment classDefinition) includes: type) ifFalse: [ ^ nil ].
	methodClassName _ class substrings
		ifEmpty: [ ^ nil ]
		ifNotEmpty: [ :parts |
			parts first asSymbol ].
	class doWithEnvironmentIfAware: [ :environment |
		methodClassName _ methodClassName forEnvironment: environment ].
	(Smalltalk includesKey: methodClassName) ifFalse: [ ^ nil ].
	methodClass _ Smalltalk at: methodClassName.
	^ meta
		ifTrue: [ methodClass class ]
		ifFalse: [ methodClass ]! !

    
!ChangeList methodsFor: 'scanning' stamp: 'KLG 10/20/2020 15:15:23'!
classDefinitionRecordFrom: tokens
	
	| classDefinition isMeta itemPosition className record fullClassName category stamp |
	
	itemPosition _ file position.
	fullClassName _ self scanClassDenotation: tokens second.
	isMeta _ fullClassName includesSubString: ' class'.
	className _ isMeta ifTrue: [fullClassName substrings first] ifFalse: [fullClassName].
	category _ CompiledMethod classCategoryFrom: tokens.
	stamp _ CompiledMethod stampFrom: tokens.
	classDefinition _ file nextChunk.
	
	record _ ChangeRecord new 
		file: file 
		position: itemPosition 
		type: #classDefinition 
		class: className asSymbol 
		category: category 
		meta: isMeta 
		stamp: stamp.
		
	record markAsTest: (classDefinition beginsWith: TestCase name asPlainString).
	
	^record
! !

!ChangeList methodsFor: 'scanning' stamp: 'KLG 10/20/2020 15:49:02'!
scanClassComment: tokens

	| className stamp record |
	
	className _ self scanClassDenotation: tokens first.
	stamp _ tokens third.
	record _ ChangeRecord new 
			file: file 
			position: file position 
			type: #classComment 
			class: className 
			category: nil 
			meta: false 
			stamp: stamp.
			
	self 
		addItem: record
		text: 'class comment for ' , className, (stamp isEmpty ifTrue: [''] ifFalse: ['; ' , stamp]).
		
	file nextChunk.
! !

!ChangeList methodsFor: 'scanning' stamp: 'KLG 10/19/2020 13:38:15'!
scanMethodDefinition: tokens

	| stamp className priorMethod overridenMethod |
	
	className _ self scanClassDenotation: tokens first.
	stamp _ CompiledMethod stampFrom: tokens.
	priorMethod _ CompiledMethod priorReferenceFrom: tokens.
	overridenMethod _ CompiledMethod overridenMethodReferenceFrom: tokens.
	
	tokens second == #methodsFor: ifTrue: [
		^ self scanCategory: tokens third class: className meta: false stamp: stamp prior: priorMethod overrides: overridenMethod ].

	tokens third == #methodsFor: ifTrue: [
		^ self scanCategory: tokens fourth class: className meta: true stamp: stamp prior: priorMethod overrides: overridenMethod ].
	
	self error: 'Unsupported method definition' 
! !

!CodePackage methodsFor: 'saving' stamp: 'KLG 11/5/2020 19:35:40'!
write: classes initializersOn: aStream
	"Write the call to #initialize method of classes defined in us."

	classes do: [ :class |
		(class class includesSelector: #initialize) ifTrue: [
			aStream nextChunkPut: class safeDenotingExpression, ' initialize'; newLine ]]! !

!Encoder methodsFor: 'results' stamp: 'KLG 11/6/2020 20:06:24'!
associationForClass
	"Answer the association of the class we compile the method for.
	
	The answer is used to be stored in method's last literal."
	| assoc |
	assoc _ Smalltalk associationAt: class name ifAbsent: [ nil ].
	^assoc value == class
		ifTrue: [ assoc ]
		ifFalse: [ | answer |
			answer _ Association new value: class.
			"Do the right thing even we do not find the method."
			class isMeta ifFalse: [ answer key: class name ].
			answer ]! !


!Object reorganize!
('Breakpoint' break)
('accessing' addInstanceVarNamed:withValue: at: at:put: basicAt: basicAt:put: basicSize customizeExplorerContents rawBasicAt: rawBasicAt:put: size yourself)
('as yet unclassified' revisar)
('associating' ->)
('binding' bindingOf:)
('casing' caseOf: caseOf:otherwise:)
('class membership' class isKindOf: isMemberOf: respondsTo:)
('comparing' = ~= hash literalEqual:)
('converting' adaptToFloat:andSend: adaptToFraction:andSend: adaptToInteger:andSend: as: asString complexContents copyAs: mustBeBoolean mustBeBooleanIn: withoutListWrapper)
('copying' copy copyForClipboard copyFrom: copySameFrom: postCopy shallowCopy veryDeepCopy)
('events-old protocol' addDependent: breakDependents changed changed: removeDependent: update:)
('error handling' assert: assert:description: caseError doesNotUnderstand: error: halt halt: handles: notYetImplemented notify: notify:at: primitiveFail primitiveFailed primitiveFailed: shouldBeImplemented shouldNotHappen shouldNotHappenBecause: shouldNotHappenBecauseErrorMessage shouldNotHappenErrorMessage shouldNotImplement subclassResponsibility)
('evaluating' value valueWithArguments: valueWithPossibleArgument:)
('events-accessing' actionForEvent: actionMap actionSequenceForEvent: actionsDo: hasActionForEvent: setActionSequence:forEvent: updateableActionMap)
('events-registering' when:evaluate: when:send:to: when:send:to:with: when:send:to:withArguments:)
('events-removing' releaseActionMap removeAction:forEvent: removeActionsForEvent: removeActionsSatisfying: removeActionsSatisfying:forEvent: removeActionsWithReceiver: removeActionsWithReceiver:forEvent:)
('events-triggering' triggerEvent: triggerEvent:with: triggerEvent:withArguments:)
('finalization' actAsExecutor executor finalizationRegistry finalize retryWithGC:until: toFinalizeSend:to:with:)
('inspecting' basicInspect copyToClipboard inspect inspectWithLabel: inspectorClass)
('macpal' flash)
('message handling' argumentName argumentNameSufix disableCode: executeMethod: perform: perform:with: perform:with:with: perform:with:with:with: perform:withArguments: perform:withArguments:inSuperclass: perform:withPossiblyWrongSizedArguments: with:executeMethod: with:with:executeMethod: with:with:with:executeMethod: with:with:with:with:executeMethod: withArgs:executeMethod:)
('object serialization' comeFullyUpOnReload: convertToCurrentVersion:refStream: objectForDataStream: readDataFrom:size: storeDataOn:)
('printing' displayStringOrText fullPrintString isLiteral longPrintOn: longPrintOn:limitedTo:indent: longPrintString longPrintStringLimitedTo: nominallyUnsent: print printAs:limitedTo: printOn: printString printStringLimitedTo: printText printTextLimitedTo: printWithClosureAnalysisOn: storeOn: storeString)
('system primitives' becomeForward: becomeForward:copyHash: className instVarAt: instVarAt:put: instVarAtPrim73: instVarAtPrim74:put: instVarNamed: instVarNamed:put: primitiveChangeClassTo: someObject)
('stepping' stepAt: wantsSteps)
('testing' is: isArray isBehavior isBlock isClosure isCollection isComplex isContext isFloat isFloatOrFloatComplex isFraction isInteger isInterval isMethodProperties isNumber isObject isPoint isPseudoContext isString isSymbol isVariableBinding name renameTo:)
('translation support' inline: success: var:declareC:)
('user interface' browseClassHierarchy confirm: confirm:orCancel: explore hasContentsInExplorer inform: request:do: request:initialAnswer: request:initialAnswer:do: request:initialAnswer:do:orCancel: request:initialAnswer:orCancel: request:initialAnswer:verifying:do: request:initialAnswer:verifying:do:orCancel:)
('private' deprecatedMethod errorDescriptionForSubcriptBounds: errorImproperStore errorNonIntegerIndex errorNotIndexable errorSubscriptBounds: primitiveError: setPinned: species storeAt:inTempFrame:)
('tracing' inboundPointers inboundPointersExcluding: outboundPointers outboundPointersDo:)
('morphic' activeHand runningWorld)
('user interface support' autoCompleterClassFor: editorClassFor: textStylerClassFor:)
('profiler support' profilerFriendlyCall:)
('pinning' isPinned pin unpin)
('debugging-haltOnce' clearHaltOnce haltOnce haltOnceEnabled setHaltOnce toggleHaltOnce)
('environment-support')
!

!classDefinition: #HierarchyBrowser category: #'Tools-Browser'!
Browser subclass: #HierarchyBrowser
	instanceVariableNames: 'classList centralClass classNamesList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!
!classDefinition: #ProtocolBrowser category: #'Tools-Browser'!
MessageSet subclass: #ProtocolBrowser
	instanceVariableNames: 'baseClass selectiveClassList selectiveClassListIndex selectiveClassNameList selectedName exclude'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!ClassDescription reorganize!
('accessing' classVersion comment comment: comment:stamp: hasComment theMetaClass theNonMetaClass)
('accessing class hierarchy' classesThatImplementAllOf: removeUninstantiatedSubclassesSilently subclasses)
('accessing method dictionary' addAndClassifySelector:withMethod:inProtocol:notifying: allMethodCategoriesIntegratedThrough: changeRecordsAt: induceMDFault methodsInCategory: recoverFromMDFault recoverFromMDFaultWithTrace removeCategory: removeSelector: removeSelectorIfInBaseSystem:)
('compiling' acceptsLoggingOfCompilation compile:classified: compile:classified:notifying: compile:classified:withStamp:notifying: compile:classified:withStamp:notifying:logSource: compile:notifying: compileSilently:classified: compileSilently:classified:notifying: instVarNamesAndOffsetsDo: isOkToAddMethod:isMeta: isOkToChangeMethod:isMeta: isOkToRemoveMethod:isMeta: localBindingOf: moveInstVarNamed:to:after: wantsChangeSetLogging wantsRecompilationProgressReported)
('copying' copy:from: copy:from:classified: copyAll:from: copyAll:from:classified: copyAllCategoriesFrom: copyCategory:from: copyCategory:from:classified: copyMethodDictionaryFrom:)
('fileIn/Out' classComment: classComment:stamp: commentFollows commentStamp: commentStamp:prior: definition definitionPreamble definitionPreambleWithoutStamp fileOutCategory: fileOutCategory:on:moveSource:toFile: fileOutChangedMessages:on: fileOutChangedMessages:on:moveSource:toFile: fileOutMethod: fileOutOn: fileOutOn:moveSource:toFile: fileOutOrganizationOn: fileOutOrganizationOn:excludingExtensions: methods methodsFor: methodsFor:priorSource:inFile: methodsFor:stamp: methodsFor:stamp:prior: moveChangesTo: printAdditionalDefinitionOn: printCategoryChunk:on: printCategoryChunk:on:priorMethod: printCategoryChunk:on:withStamp:priorMethod: printCategoryChunk:on:withStamp:priorMethod:overridesMethod: printCategoryChunk:withStamp:on: printMethodChunk:withPreamble:on:moveSource:toFile: putClassCommentToCondensedChangesFile: reformatAll reformatMethodAt: reorganize)
('initialization' doneCompiling superclass:methodDictionary:format: updateInstancesFrom:)
('initialize-release' updateInstances:from:isMeta: updateMethodBindingsTo:)
('instance variables' addInstVarName: allInstVarNamesEverywhere browseClassVarRefs checkForInstVarsOK: chooseClassVarName chooseDefiningInstanceVariableAlphabeticallyWith:thenDo: chooseDefiningInstanceVariableAlphabeticallyWith:thenDo:ifNone: chooseInstVarAlphabeticallyThenDo: chooseInstVarThenDo: classThatDefinesClassVariable: classThatDefinesInstanceVariable: forceNewFrom: instVarNameForIndex: instVarNames removeInstVarName: withClassesThatDefineInHierarchyInstanceVariable:do:ifNone: withSubclassesThatDefineInstanceVariable:do:ifNone:)
('method dictionary' allMethodsInCategory:)
('organization' category category: organization organization: whichCategoryIncludesSelector: zapOrganization)
('printing' classVariablesString denotingExpression instanceVariablesString printOn: printWithClosureAnalysisOn: safeDenotingExpression sharedPoolsString storeOn:)
('release' obsolete)
('system-support' allUnreferencedClassVariables)
('testing' hasDefinition)
('private' errorCategoryName instVarMappingFrom: linesOfCode logMethodSource:forMethodWithNode:inCategory:withStamp:notifying: newInstanceFrom:variable:size:map: setInstVarNames:)
('accessing instances and variables' allUnreferencedInstanceVariables)
('pool variables' classPool)
!


!Class reorganize!
('accessing' name)
('testing' hasMethods isObsolete)
('copying' copy)
('class name' rename: safeRenameTo:)
('instance variables' addInstVarName: removeInstVarName:)
('class variables' addClassVarName: allClassVarNames classVarNames ensureClassPool removeClassVarName:)
('pool variables' addSharedPool: allSharedPools classPool classPool: classPoolFrom: removeSharedPool: sharedPools)
('compiling' binding bindingOf: compileAllFrom: localBindingOf: nonLocalBindingOf: possibleVariablesFor:continuedFrom:)
('environment support')
('subclass creation' subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:inSimpleEnvironment: immediateSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category: subclass:instanceVariableNames:classVariableNames:poolDictionaries:category: variableByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category: variableDoubleByteSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category: variableDoubleWordSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category: variableSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category: variableWordSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category: weakSubclass:instanceVariableNames:classVariableNames:poolDictionaries:category:)
('fileIn/Out' definitionReplacingCategoryWith: fileOut fileOutInitializerOn: fileOutOn:moveSource:toFile: fileOutOn:moveSource:toFile:initializing: fileOutPool:onFileStream: fileOutSharedPoolsOn: objectForDataStream: printAdditionalDefinitionOn: reformatAll shouldFileOutPool: shouldFileOutPools storeDataOn:)
('accessing class hierarchy' addSubclass: removeSubclass: subclasses subclassesDo: subclassesDoGently:)
('private' becomeCompact becomeCompactSimplyAt: becomeUncompact setName: spaceUsed)
('initialization' declare: sharing: superclass:methodDict:format:name:organization:instVarNames:classPool:sharedPools: superclass:methodDictionary:format:)
('release' obsolete removeFromSystem removeFromSystem: removeFromSystemUnlogged unload)
('tdd' testCaseClass)
('printing' denotingExpression safeDenotingExpression)
!


!Metaclass reorganize!
('accessing' allInstances category isMeta name soleInstance theMetaClass theNonMetaClass)
('copying' postCopy)
('instance creation' new)
('instance variables' addInstVarName: removeInstVarName:)
('pool variables' classPool)
('class hierarchy' addObsoleteSubclass: addSubclass: obsoleteSubclasses removeObsoleteSubclass: removeSubclass: subclasses subclassesDo: subclassesDoGently:)
('compiling' acceptsLoggingOfCompilation bindingOf: forNonLocalBindingEnvironmentsDo: localBindingOf: possibleVariablesFor:continuedFrom: wantsChangeSetLogging wantsRecompilationProgressReported)
('fileIn/Out' definition fileOutInitializerOn: fileOutOn:moveSource:toFile: fileOutOn:moveSource:toFile:initializing: nonTrivial objectForDataStream: storeDataOn:)
('testing' canZapMethodDictionary isObsolete)
('enumerating' allInstancesDo:)
('private' replaceObsoleteInstanceWith:)
('initialization' adoptInstance:from: instanceVariableNames:)
('tdd' testCaseClass)
('accessing instances and variables' allClassVarNames)
('class name')
('printing' denotingExpression denotingExpressionForClass safeDenotingExpressionForClass)
!


!String reorganize!
('accessing' at: at:put: byteAt: byteAt:put: findAnySubStr:startingAt: findBetweenSubStrs: findDelimiters:startingAt: findLastOccurrenceOfString:startingAt: findString: findString:startingAt: findString:startingAt:caseSensitive: findTokens: findTokens:keep: includesSubString: includesSubstring:caseSensitive: indexOf: indexOf:startingAt: indexOf:startingAt:ifAbsent: indexOfAnyOf: indexOfAnyOf:startingAt:ifAbsent: indexOfSubCollection:startingAt:ifAbsent: lineCount lineIndicesDo: lineNumber: lines linesDo: skipAnySubStr:startingAt: skipDelimiters:startingAt: string)
('comparing' < <= = > >= alike: beginsWith: caseInsensitiveLessOrEqual: caseSensitiveLessOrEqual: commonPartWith:startAt:stopAt:applying: commonPrefixWith: compare: compare:caseSensitive: crc16 endsWith: hash is:substringAt: match: sameAs: startingAt:match:startingAt:)
('copying' , append: appendToString: appendToText: copyReplaceTokens:with: copyReplacing: copyReplacing:into: copyReplacing:with: copyReplacing:with:into: flattenTo: join: newFromAfterAppending:into:keepingFrom:skipping: padded:to:with:)
('converting' adaptToCollection:andSend: adaptToNumber:andSend: adaptToPoint:andSend: afterBlanksEndsWith: asCamelCase asDate asFileName asHex asIdentifier: asInteger asLowercase asMonth asNumber asSmalltalkComment asString asSymbol asText asUnHtml asUnaccented asUppercase asUtf8 asUtf8: asValidInstanceVariableName asValidSelector asWeek asYear base64Decoded capitalized contractTo: correctAgainst: correctAgainst:continuedFrom: correctAgainstDictionary:continuedFrom: displayStringOrText findSelector firstNoBlankIndex initialIntegerOrNil keywords lastNoBlankIndex prefixAndSuffix: romanNumber squeezedTo: substrings surroundedBySingleQuotes translateFrom:to:table: translateToLowercase translateToUppercase translateWith: truncateTo: truncateWithElipsisTo: uncapitalized withBlanksCondensed withBlanksTrimmed withCuisLineEndings withDescriptiveLineEndings withLineEndings: withNewLines withoutEnclosing: withoutLeadingBlanks withoutPrefix: withoutSeparators withoutSuffix: withoutTrailingBlanks)
('enumerating' collect:)
('displaying' displayAt: displayOn: displayOn:at: displayOn:at:textColor: displayProgressAt:from:to:during:)
('printing' isLiteral print printOn: storeOn:)
('private' correctAgainstEnumerator:continuedFrom: evaluateExpression:parameters: getEnclosedExpressionFrom: replaceFrom:to:with:startingAt:)
('system primitives' compare:with:collated: findSubstring:in:startingAt:matchTable: numArgs)
('testing' hasContentsInExplorer is: isAlphaNumeric isRemote isString)
('paragraph support' encompassLine: encompassParagraph: endOfParagraphBefore: indentationIfBlank:)
('arithmetic' * + - / // \\)
('user interface' edit editLabel:)
('formatting' format:)
('uCompletion' separateKeywords)
('fileman-converting' asAbsolutePathName asDirectoryEntry asDriveName asFileEntry asPathComponents asPathTokens withoutWindowsDriveName)
('fileman-testing' beginsWithPathSeparator beginsWithWindowsDriveName isAbsolutePathName isDriveName isPathSeparator isRelativeMark isRelativePathName)
('fileman-actions' fileContents fileContents:)
('fileman-accessing' indexOfFirstPathSeparator indexOfLastPathSeparator pathAndLocalName upToFirstPathSeparator upToLastPathSeparator)
('text conversion emphasis' bold italic pointSize: struck sub super under)
('text conversion attributes' centered justified leftFlush rightFlush)
('text conversion color' black blue cyan gray green magenta red yellow)
('grammar' article asPlural exceptionalPlural isUninflictedNoun lowercasePlural pluralize: withArticle)
!


!Symbol reorganize!
('accessing' at:put: precedence replaceFrom:to:with:startingAt:)
('comparing' =)
('copying' shallowCopy)
('converting' asString asSymbol capitalized)
('printing' print storeOn:)
('system primitives' flushCache numArgs:)
('private' errorNoModification species string:)
('testing' is: isInfix isInitializePvtSelector isKeyword isLiteral isPvtSelector isSymbol isUnary isValidBinarySelector isValidKeywordSelector isValidSelector isValidUnarySelector)
('uCompletion' separateKeywords)
!


!SystemDictionary reorganize!
('accessing' classes description name organization summary)
('class names' classNamed: fillCaches flushClassNameCache forgetClass:logged: hasClassNamed: prepareToRenameClass:as: prepareToRenameClass:from:to: removeClassNamed: removeClassNamedIfInBaseSystem: renameAndUpdateReferences:as: renameClassNamed:as: renamedClass:from:)
('dictionary access' associationOrUndeclaredAt: at:put:)
('housekeeping' browseEqEqSentToSmallIntegerConstants browseObsoleteMethodReferences browseUndeclaredReferences cleanCompactObsoleteClasses cleanOutUndeclared condenseChanges condenseSources macroBenchmark1 macroBenchmark3 obsoleteBehaviors obsoleteClasses obsoleteMethodReferences recompileAllFrom: removeEmptyMessageCategories testDecompiler testFormatter testFormatter2 verifyChanges)
('image, changes name' alternativeSourcesName changeImageNameTo: currentChangesName currentSourcesName defaultChangesName defaultSourcesName defaultUserChangesName fullNameForChangesNamed: fullNameForImageNamed: imageName imageName: imagePath primImageName primVmPath vmPath)
('memory space' bytesLeft bytesLeft: bytesLeftString createStackOverflow garbageCollect garbageCollectMost growMemoryByAtLeast: installLowSpaceWatcher lowSpaceThreshold lowSpaceWatcher lowSpaceWatcherProcess okayToProceedEvenIfSpaceIsLow primBytesLeft primLowSpaceSemaphore: primSignalAtBytesLeft: primitiveGarbageCollect signalLowSpace stopLowSpaceWatcher useUpMemory useUpMemoryWithArrays useUpMemoryWithContexts useUpMemoryWithTinyObjects)
('code authors' allContributors contributionsOf: knownInitialsAndNames unknownContributors)
('miscellaneous' cogitClass exitToDebugger extraVMMemory extraVMMemory: getCurrentWorkingDirectory getSystemAttribute: getVMParameters handleUserInterrupt interpreterClass isDevelopmentEnvironmentPresent isHeadless listBuiltinModule: listBuiltinModules listLoadedModule: listLoadedModules logError:inContext:to: maxExternalSemaphores maxExternalSemaphores: osVersion platformName platformSubtype primGetCurrentWorkingDirectoryUnix primGetCurrentWorkingDirectoryWindows primVmFileName unloadModule: vmFileName vmOptionsDo: vmVersion voidCogVMState)
('object serialization' objectForDataStream: storeDataOn:)
('printing' printElementsOn: printOn:)
('profiling' clearProfile dumpProfile profile: startProfiling stopProfiling)
('retrieving' allBehaviorsDo: allCallsOn: allCallsOn:and: allClasses allClassesDo: allClassesImplementing: allGlobalRefs allGlobalRefsWithout: allImplementedMessages allImplementedMessagesWithout: allImplementorsOf: allImplementorsOf:localTo: allMethodsInCategory: allMethodsSourceStringMatching: allMethodsWithSourceString:matchCase: allMethodsWithString: allObjects allObjectsDo: allObjectsOrNil allPrimitiveMessages allPrimitiveMethodsInCategories: allReferencesToLiteral: allSelect: allSentMessages allSentMessagesWithout: allUnSentMessages allUnSentMessagesIn: allUnSentMessagesWithout: allUnimplementedCalls allUnusedClassesWithout: hierachySortedAllClassesDo: isThereAReferenceTo: isThereAnImplementorOf: isThereAnImplementorOf:exceptFor: numberOfImplementorsOf: numberOfSendersOf: pointersTo: pointersTo:except: poolUsers unimplemented)
('shrinking' abandonSources presumedSentMessages reduceCuis removeAllUnSentMessages removeSelector: removedUnusedClassesAndMethods reportClassAndMethodRemovalsFor: unusedClasses unusedClassesAndMethodsWithout:)
('snapshot and quit' add:toList:after: addToShutDownList: addToShutDownList:after: addToStartUpList: addToStartUpList:after: lastQuitLogPosition logSnapshot:andQuit: nopTag okayToDiscardUnsavedCode okayToSave printStuffToCleanOnImageSave processShutDownList: processStartUpList: quitNoSaveTag quitPrimitive quitPrimitive: quitTag removeFromShutDownList: removeFromStartUpList: saveAs saveAs:andQuit:clearAllClassState: saveAsNewVersion saveSession send:toClassesNamedIn:with: setGCParameters setPlatformPreferences snapshot:andQuit: snapshot:andQuit:clearAllClassState: snapshot:andQuit:embedded: snapshot:andQuit:embedded:clearAllClassState: snapshotEmbeddedPrimitive snapshotMessageFor:andQuit: snapshotPrimitive snapshotTag snapshotTagFor:andQuit: storeOn: tagHeader tagTail unbindExternalPrimitives)
('sources, change log' aboutThisSystem assureStartupStampLogged calcEndianness classRemoved:fromCategory: closeSourceFiles endianness evaluated:context: externalizeSources forceChangesToDisk internalizeChangeLog internalizeSources isBigEndian isLittleEndian lastUpdateString logChange: logChange:preamble: methodRemoved:selector:inProtocol:class: openSourceFiles openSourcesAndChanges systemInformationString timeStamp: version)
('special objects' clearExternalObjects compactClassesArray compactClassesArrayIncludes: externalObjects hasSpecialSelector:ifTrueSetByte: primitiveErrorTable recreateSpecialObjectsArray registerExternalObject: specialNargsAt: specialObjectsArray specialSelectorAt: specialSelectorSize specialSelectors unregisterExternalObject:)
('toDeprecate')
('ui' beep confirmRemovalOf:on: globals inspectGlobals primitiveBeep systemCategoryFromUserWithPrompt:)
('browsing' browseAllAccessesTo:from: browseAllCallsOn: browseAllCallsOn:and: browseAllCallsOn:localTo: browseAllCallsOnClass: browseAllImplementorsOf: browseAllImplementorsOf:localTo: browseAllImplementorsOfList: browseAllImplementorsOfList:title: browseAllMethodsInCategory: browseAllPrimitives browseAllReferencesToLiteral: browseAllSelect: browseAllSelect:name:autoSelect: browseAllStoresInto:from: browseAllUnSentMessages browseAllUnimplementedCalls browseClassCommentsWithString: browseClassesWithNamesContaining:caseSensitive: browseEqSmallConstant browseInstVarDefs: browseInstVarRefs: browseLikelyUnnededRedefinitions browseMessageList:name: browseMessageList:name:autoSelect: browseMessageList:ofSize:name:autoSelect: browseMethodsWithSourceString: browseMethodsWithString: browseMethodsWithString:matchCase: browseMyChanges browseObsoleteReferences browseViewReferencesFromNonViews showMenuOf:withFirstItem:ifChosenDo: showMenuOf:withFirstItem:ifChosenDo:withCaption:)
('private' allSymbolsIn:do: baseLabel)
('Closure measurements' browseMethodsWithClosuresThatAccessOuterTemps browseMethodsWithClosuresThatOnlyReadOuterTemps browseMethodsWithClosuresThatWriteOuterTemps browseMethodsWithClosuresThatWriteOuterTempsButCleanOtherwise browseMethodsWithEmptyClosures browseMethodsWithMoreThanOneClosure browseMethodsWithOnlyCleanClosures closuresInfoStringForClass:selector: eliotsClosureMeasurements eliotsClosureMeasurements2 eliotsClosureMeasurements2On: eliotsClosureMeasurementsOn:over:)
('removing' removeKey:ifAbsent:)
('system attributes' flagInterpretedMethods: isRunningCog isRunningCogit isSpur maxIdentityHash processHasThreadIdInstVar: processPreemptionYields processPreemptionYields:)
('query' hierarchySorted:do: methodsWithUnboundGlobals unusedBlocks)
('testing' hasBindingThatBeginsWith: isLiveSmalltalkImage isLiveTypingInstalled)
('classes and traits' classNames nonClassNames)
('image format' imageFormatVersion imageFormatVersionFromFile imageFormatVersionFromFileAsIs)
('AndreasProfiler-profiling' interruptChecksPerMSec longRunningPrimitive longRunningPrimitiveSemaphore: profilePrimitive profileSample profileSemaphore: profileStart:)
('startup' doStartUp: processCommandLineArgument:storeStartUpScriptArgsOn: processCommandLineArguments readCommandLineArguments setStartupStamp startUpArguments startUpScriptArguments)
('startup - restore lost changes' hasToRestoreChanges hasToRestoreChangesFrom: isQuitNoSaveRecord: isQuitRecord: isSnapshotQuitOrQuitNoSaveRecord: isSnapshotRecord: lostChangesDetectedCaption restoreLostChanges restoreLostChangesAutomatically restoreLostChangesAutomaticallyFrom: restoreLostChangesIfNecessary restoreLostChangesManually restoreLostChangesOptions restoringChangesHasErrorsCaption shouldShowFileInErrors withChangesFileDo:)
('image' wordSize)
('vm parameters' doMixedArithmetic doMixedArithmetic: vmParameterAt: vmParameterAt:put:)
('environment support')
!


!CodeFileBrowser class reorganize!
('instance creation' browseCodeFileEntry: browseCodeSource: browseCodeSource:base: browsePackageFileEntry: fileReaderServicesForFileEntry:suffix: serviceBrowseCode serviceBrowsePackage)
('scanning' scanClassDenotation:)
!
