'From Haver 5.0 [latest update: #4572] on 4 May 2021 at 5:32:16 pm'!
'Description I implement a distributed hash table as specified by the Kademlia standard.'!
!provides: 'Kademlia' 1 1!
!requires: 'Identities-UUID' 1 8 nil!
!requires: 'Cryptography-DigitalSignatures' 1 14 nil!
SystemOrganization addCategory: 'Kademlia'!


!moduleCreation: Kademlia!
Modules newEnvironment: #Kademlia!

!interfacesOf: Kademlia!
Modules environment: #Kademlia ::
	interface: #SPI aliasFor: #API!


!classDefinition: (Modules>>#Kademlia>>#Configuration) category: 'Kademlia'!
Object subclass: #Configuration
	instanceVariableNames: 'numberOfConcurrentRequests secureHashAlgorithm keySize bucketSize expirationTime refreshTime replicationTime republicationTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kademlia'
	inModule: #Kademlia!
!classDefinition: (Modules>>#Kademlia>>#Configuration) class category: 'Kademlia'!
(Modules>>#Kademlia>>#Configuration) class
	instanceVariableNames: 'default'!

!classDefinition: (Modules>>#Kademlia>>#Node) category: 'Kademlia'!
Object subclass: #Node
	instanceVariableNames: 'id adresses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kademlia'
	inModule: #Kademlia!
!classDefinition: (Modules>>#Kademlia>>#Node) class category: 'Kademlia'!
(Modules>>#Kademlia>>#Node) class
	instanceVariableNames: ''!


!(Modules>>#Kademlia>>#Configuration) commentStamp: '<historical>' prior: 0!
The orginal name uses other names for the parameters that are not helpful:

numberOfConcurrentRequests -> alpha
keySize ->  B (computed from secureHashAlgorithm)
k -> bucketSize
!

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/3/2021 19:40:30'!
defaultBucketSize
	"Answer the default bicket size."

	^ 20! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/3/2021 19:43:04'!
defaultExpirationTime
	"Abswer the default expiration time."

	^ Duration days: 1! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/3/2021 19:45:05'!
defaultRefreshTime
	"Abswer the default refresh time."

	^ Duration hours: 1! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/3/2021 19:45:53'!
defaultReplicationTime
	"Abswer the default replication time."

	^ Duration hours: 1! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/3/2021 19:48:12'!
defaultRepublicationTime
	"Abswer the default republication time."

	^ Duration days: 1! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/3/2021 19:51:38'!
defaultSecureHashAlgorithm
	"Abswer the default secure hash algorithm.."

	^ SecureHashAlgorithm256 ! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:52'!
bucketSize
	"Answer the value of bucketSize"

	^ bucketSize ifNil: [ self defaultBucketSize ]! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
bucketSize: anObject
	"Set the value of bucketSize"

	bucketSize _ anObject! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:39:45'!
expirationTime
	"Answer the value of expirationTime"

	^ expirationTime ifNil: [ self defaultExpirationTime ]! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
expirationTime: anObject
	"Set the value of expirationTime"

	expirationTime _ anObject! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 20:01:08'!
keySize
	"Answer the value of keySize"

	^ keySize ifNil: [
		self secureHashAlgorithm new hashMessage: 'Urmel aus dem Eis' :: 
			size << 3]! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
numberOfConcurrentRequests
	"Answer the value of numberOfConcurrentRequests"

	^ numberOfConcurrentRequests! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
numberOfConcurrentRequests: anObject
	"Set the value of numberOfConcurrentRequests"

	numberOfConcurrentRequests _ anObject! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
refreshTime
	"Answer the value of refreshTime"

	^ refreshTime! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
refreshTime: anObject
	"Set the value of refreshTime"

	refreshTime _ anObject! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
replicationTime
	"Answer the value of replicationTime"

	^ replicationTime! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
replicationTime: anObject
	"Set the value of replicationTime"

	replicationTime _ anObject! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
republicationTime
	"Answer the value of republicationTime"

	^ republicationTime! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:38:07'!
republicationTime: anObject
	"Set the value of republicationTime"

	republicationTime _ anObject! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:51:58'!
secureHashAlgorithm
	"Answer the value of secureHashAlgorithm"

	^ secureHashAlgorithm ifNil: [ self defaultSecureHashAlgorithm ]! !

!(Modules>>#Kademlia>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 19:49:33'!
secureHashAlgorithm: anObject
	"Set the value of secureHashAlgorithm"

	secureHashAlgorithm _ anObject.
	keySize _ nil! !

!(Modules>>#Kademlia>>#Configuration) class methodsFor: 'defaults' stamp: 'KLG 5/3/2021 21:01:44'!
default
	"Answer the default configuration"

	^ default ifNil: [ default _ self new ]! !

!(Modules>>#Kademlia>>#Node) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 21:26:50'!
addAddress: anAddress
	"Add an address to our list of addresses."

	adresses add: anAddress! !

!(Modules>>#Kademlia>>#Node) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 21:26:50'!
addressList
	"Answer the value of addressList"

	^ adresses! !

!(Modules>>#Kademlia>>#Node) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 21:12:29'!
id
	"Answer the value of id"

	self assert: id notNil.
	^ id! !

!(Modules>>#Kademlia>>#Node) methodsFor: 'accessing' stamp: 'KLG 5/3/2021 21:12:14'!
id: anObject
	"Set the value of id"

	self assert: id isNil.
	id _ anObject! !

!(Modules>>#Kademlia>>#Node) methodsFor: 'initialization' stamp: 'KLG 5/3/2021 21:26:50'!
initialize
	"Initialize the address list."

	super initialize.
	adresses _ Set new: 1! !

!(Modules>>#Kademlia>>#Node) class methodsFor: 'instance creation' stamp: 'KLG 5/3/2021 21:10:26'!
ownNode
	"Answer a node intialized for us."

	^ self new id: (Configuration default secureHashAlgorithm new hashMessage: UUID new)! !
