'From Haver 6.0 [latest update: #5213] on 1 June 2022 at 6:08:32 pm'!
'Description '!
!provides: 'Distributor' 1 36!
!requires: 'FileFinder' 1 21 nil!
!requires: 'Compression' 1 30 nil!
SystemOrganization addCategory: 'Distributor'!


!moduleCreation: Distributor!
Modules newEnvironment: #Distributor!

!interfacesOf: Distributor!
Modules environment: #Distributor ::
	interface: #SPI aliasFor: #API ::
	interface: #UTI aliasFor: #API!

!importsOf: Distributor!
Modules environment: #Distributor :: 
	import: #API of: Modules>>#FileFinder!

!classDefinition: (Modules>>#Distributor>>#AbstractDistributor) category: 'Distributor'!
Object subclass: #AbstractDistributor
	instanceVariableNames: 'filefinder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Distributor'
	inModule: #Distributor!
!classDefinition: (Modules>>#Distributor>>#AbstractDistributor) class category: 'Distributor'!
(Modules>>#Distributor>>#AbstractDistributor) class
	instanceVariableNames: ''!

!classDefinition: (Modules>>#Distributor>>#ZipFileDistributor) category: 'Distributor'!
(Modules>>#Distributor>>#AbstractDistributor) subclass: #ZipFileDistributor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Distributor'
	inModule: #Distributor!
!classDefinition: (Modules>>#Distributor>>#ZipFileDistributor) class category: 'Distributor'!
(Modules>>#Distributor>>#ZipFileDistributor) class
	instanceVariableNames: ''!

!classDefinition: (Modules>>#Distributor>>#Configuration) category: 'Distributor'!
Object subclass: #Configuration
	instanceVariableNames: 'cuisRootDirectory haverRootDirectory environmentRootDirectory thirdPartyDirectories explicitUpdateNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Distributor'
	inModule: #Distributor!
!classDefinition: (Modules>>#Distributor>>#Configuration) class category: 'Distributor'!
(Modules>>#Distributor>>#Configuration) class
	instanceVariableNames: 'default'!


!(Modules>>#Distributor>>#AbstractDistributor) commentStamp: '<historical>' prior: 0!
I am the abstract base class of all distributors.!

!(Modules>>#Distributor>>#ZipFileDistributor) commentStamp: '<historical>' prior: 0!
I am the zip file distributer.!

!(Modules>>#Distributor>>#Configuration) commentStamp: '<historical>' prior: 0!
I contain the configuration for the distributor.!

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'instance creation' stamp: 'KLG 4/9/2021 19:36:56'!
fileFinderClass
	"Answer the file finder class"

	^ self class fileFinderClass! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'roots' stamp: 'KLG 4/11/2021 15:13:17'!
cuisRootDirectory
	"Answer the cuis root directory."
	
	^ Configuration current cuisRootDirectory! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'roots' stamp: 'KLG 4/11/2021 15:13:23'!
environmentRootDirectory
	"Answer the root directory for the environment change set."
	
	^ Configuration current environmentRootDirectory! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'roots' stamp: 'KLG 4/11/2021 15:13:29'!
haverRootDirectory
	"Answer the haver root directory."
	
	^ Configuration current haverRootDirectory! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'roots' stamp: 'KLG 4/12/2021 22:54:48'!
thirdPartyDirectories
	"Answer all the third party directories."
	
	^ Configuration current thirdPartyDirectories! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'initialization' stamp: 'KLG 4/9/2021 19:38:56'!
initialize
	"Initialize the file finder"

	filefinder _ self fileFinderClass new.
	self initializeFileFinder! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'initialization' stamp: 'KLG 4/21/2022 21:59:53'!
initializeFileFinder
	"Initialize the file finder"

	filefinder
		cuisRoot: self cuisRootDirectory
		andHaverRoot: self haverRootDirectory 
		andEnvironmentsCoreChanges: self environmentRootDirectory.
	filefinder 
		cuisImageFileFinder explicitUpdateNumber:  self explicitUpdateNumber.
	filefinder 
		cuisChangesFileFinder explicitUpdateNumber:  self explicitUpdateNumber.
	self thirdPartyDirectories do: [ :directory |
		filefinder packageFileFinder addRoot: directory.
		filefinder licenseFileFinder addRoot: directory.
		filefinder documentationFileFinder addRoot: directory ]! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'distribution' stamp: 'KLG 4/9/2021 19:48:04'!
distribute
	"Distribute the haver stuff."

	self subclassResponsibility ! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'explicit system version' stamp: 'KLG 4/15/2021 14:37:45'!
explicitUpdateNumber
	"Answer an explict highest update number."

	^ Configuration current explicitUpdateNumber! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'versioning' stamp: 'KLG 4/25/2022 13:17:07'!
printVersionStringOn: aStream
	"Answer a version string for the distribution."
	
	| cc |
	aStream 
		print: (cc _ Configuration current) versionNumber.
	cc versionSuffix ifNotNil: [ :suffix |
		aStream
			nextPut: $-;
			nextPutAll: suffix ].
	aStream
		nextPut: $-;
		print: SystemVersion current highestUpdate! !

!(Modules>>#Distributor>>#AbstractDistributor) methodsFor: 'versioning' stamp: 'KLG 4/22/2021 23:04:43'!
versionString
	"Answer a version string for the distribution."
	
	^ String streamContents: [ :stream | 		self printVersionStringOn: stream ]! !

!(Modules>>#Distributor>>#AbstractDistributor) class methodsFor: 'instance creation' stamp: 'KLG 4/9/2021 19:36:38'!
fileFinderClass
	"Answer the file finder class"

	^ ComplexFileFinder ! !

!(Modules>>#Distributor>>#AbstractDistributor) class methodsFor: 'distribution' stamp: 'KLG 4/14/2021 20:27:34'!
distribute
	"Distribute the haver stuff."

	^ self new distribute! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 6/1/2022 18:07:50'!
addStartBatchFor: anImageFilename cuisImageFilename: aCuisImageFilename toArchive: aZipArchive
	"Add a start script for the image filename given."

	| batchName |
	anImageFilename		beginsWith: 'Image\Haver' :: ifTrue: [		batchName _ 'haver.bat' ].
	anImageFilename 		beginsWith: 'Image\Cuis' :: ifTrue: [ batchName _ 'cuis.bat' ].
	batchName 	ifNil: [ ^ self ].
	aZipArchive 
		addString:
			(String streamContents: [ :scriptStream |
				aCuisImageFilename ifNotNil: [ 
					batchName = 'haver.bat' ifTrue: [
						scriptStream
							nextPutAll: 'IF NOT EXIST ';
							nextPutAll: anImageFilename;
							nextPutAll: ' VM\SqueakConsole -headless ';
							nextPutAll: aCuisImageFilename;
							nextPutAll: ' -r Haverize --integrate';
							newLine ] ].
				scriptStream
					nextPutAll: 'VM\SqueakConsole ';
					nextPutAll: anImageFilename ;
					nextPutAll: ' %*' ])
		as: batchName! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 4/22/2022 10:45:44'!
addStartBatchFor: anImageFilename toArchive: aZipArchive
	"Add a start script for the image filename given."

	self addStartBatchFor: anImageFilename cuisImageFilename: nil toArchive: aZipArchive! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 5/29/2022 14:49:11'!
addStartScriptFor: anImageFilename cuisImageFilename: aCuisImageFilename toArchive: aZipArchive
	"Add a start script for the image filename given."

	| scriptName |
	anImageFilename		beginsWith: 'Image/Haver' :: ifTrue: [		scriptName _ 'haver' ].
	anImageFilename 		beginsWith: 'Image/Cuis' :: ifTrue: [ scriptName _ 'cuis' ].
	scriptName 	ifNil: [ ^ self ].
	aZipArchive 
		addString:
			(String streamContents: [ :scriptStream |
				scriptStream
					nextPutAll: '#!!/bin/sh';
					newLine.
				aCuisImageFilename ifNotNil: [ 
					scriptName = 'haver' ifTrue: [
						scriptStream
							nextPutAll: 'test -f $(dirname $0)/';
							nextPutAll: anImageFilename;
							nextPutAll: ' || $(dirname $0)/VM/squeak -vm-sound-null -vm-display-null -headless $(dirname $0)/';
							nextPutAll: aCuisImageFilename;
							nextPutAll: ' -r Haverize --integrate';
							newLine ] ].
				scriptStream 
					nextPutAll: 'exec $(dirname $0)/VM/squeak $(dirname $0)/';
					nextPutAll: anImageFilename ;
					nextPutAll: ' $*' ])
		as: scriptName :: unixFileAttributes: self unixFileAttributesForExecutables! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 4/21/2022 21:56:24'!
addStartScriptFor: anImageFilename toArchive: aZipArchive
	"Add a start script for the image filename given."

	self addStartScriptFor: anImageFilename cuisImageFilename: nil toArchive: aZipArchive! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 4/22/2022 10:45:44'!
distribute
	"Distribute the haver stuff."

	| zipArchive licenseFileString imageFilenames cuisImageFileName haverImageSeen |
	imageFilenames _ OrderedCollection new: 2.
	(zipArchive _ ZipArchive new) zipFileComment: 'Haver on Cuis'.
	haverImageSeen _ false.
	licenseFileString _ String streamContents: [ :licenseTextsStream |
		licenseTextsStream
			nextPutAll: 'LICENSES';
			newLine; newLine;
			nextPutAll: 'This file contains a copy of all the licenses found.';
			newLine; newLine; newLine.
		filefinder bundlesDo: [ :fileBundle |
			fileBundle isImage ifTrue:[ | dfn |
				(dfn _ fileBundle distributionFilename) includesSubString: 'Cuis' :: ifTrue: [
					cuisImageFileName _ dfn ].
				dfn includesSubString: 'Haver' :: ifTrue: [
					haverImageSeen _ true ].
				imageFilenames add: dfn ].			
			fileBundle isLicenseFile
				ifTrue: [
					licenseTextsStream
						nextPutAll: 'License from: ';
						nextPutAll: fileBundle distributionFilename;
						newLine; newLine;
						nextPutAll: fileBundle sourceFilename asFileEntry textContents;
						newLine; newLine; newLine ]
				ifFalse: [
					zipArchive
						addFile: fileBundle sourceFilename
						as: fileBundle distributionFilename ::
							unixFileAttributes: (self unixFileAttributesFor: fileBundle) ] ] ].
	zipArchive
		addString: licenseFileString
		as: 'LICENSES.txt' ::
			unixFileAttributes: self unixFileAttributesForOrdinaryFiles.
	zipArchive
		addString: 'This is an indicator for a unzipped Haver distribution.'
		as: HaverizePackage unzippedDistributionIndicatorFilename ::
			unixFileAttributes: self unixFileAttributesForOrdinaryFiles.
	imageFilenames do: [ :imageFileName |
		self 
			addStartScriptFor: imageFileName toArchive: zipArchive;
			addStartBatchFor: imageFileName toArchive: zipArchive ].
	haverImageSeen ifFalse: [ | haverImageFilename |
		self flagHaver: 'Move this stuff to the HaverImageFileFinder!!'.
		"This is a bit hack, because it duplicates information from the file finder:"
		haverImageFilename _ 'Image',
				FileIOAccessor default pathNameDelimiter asString,
				Smalltalk haverBaseNameForCurrentVersion,
				'.image'.
		self 
			addStartScriptFor: haverImageFilename
				cuisImageFilename: cuisImageFileName
				toArchive: zipArchive;
			addStartBatchFor: haverImageFilename
				cuisImageFilename: cuisImageFileName
				toArchive: zipArchive ].
	zipArchive writeToFileNamed: self zipFileName! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 4/15/2021 21:47:47'!
unixFileAttributesFor: aFileBundle
	"Answer the unix permission for aFileBundle."

	^ aFileBundle isVM 
		ifFalse: self unixFileAttributesForOrdinaryFiles 
		ifTrue: self unixFileAttributesForExecutables ! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 4/15/2021 21:46:47'!
unixFileAttributesForExecutables

	^ 8r0100775! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'distribution' stamp: 'KLG 4/15/2021 21:47:25'!
unixFileAttributesForOrdinaryFiles

	^ 8r0100664! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'filenames' stamp: 'KLG 4/16/2021 09:39:16'!
osVersion
	"Answer the os version."

	^ Smalltalk osVersion! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'filenames' stamp: 'KLG 4/16/2021 09:38:53'!
platformName
	"Answer the platform name."

	^ Smalltalk  platformName! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'filenames' stamp: 'KLG 4/15/2021 21:45:45'!
platformSubtype
	"Answer the platform subtype."

	^ Smalltalk platformSubtype! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'filenames' stamp: 'KLG 4/16/2021 09:48:54'!
vmBitSize
	"Answer number of bits the vm uses."

	^ Smalltalk wordSize << 3! !

!(Modules>>#Distributor>>#ZipFileDistributor) methodsFor: 'filenames' stamp: 'KLG 4/26/2021 12:32:35'!
zipFileName
	"Answer the zip filename"

	^ String streamContents: [ :stream |
		stream
			nextPutAll: 'Haver-';
			nextPutAll: self platformName;
			nextPut: $-;
			nextPutAll: self osVersion;
			nextPut: $-;
			nextPutAll: self platformSubtype;
			nextPut: $-;
			nextPutAll: self vmBitSize asString;
			nextPutAll: 'bit-'.
		self printVersionStringOn: stream.
		stream nextPutAll: '.zip' ]! !

!(Modules>>#Distributor>>#ZipFileDistributor) class methodsFor: 'distribution' stamp: 'KLG 4/25/2021 23:26:08'!
worldMenuOptions

	^`{{
		#itemGroup 				-> 		40.
		#itemOrder 				-> 		0.
		#label 				-> 		'Create Zip Distribution'.
		#object 				-> 		`Modules >> #Distributor >> #ZipFileDistributor`.
		#selector 				-> 		#distribute.
		#icon 				-> 		#saveIcon.
		#balloonText 				-> 		('Save a new Haver distribution as ZIP-file') withNewLines.
	} asDictionary }`! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 4/26/2021 12:02:59'!
cuisRootDirectoryDefault
	"Answer the default value value of cuisRootDirectory.
	
	Note that this is convenient for me (KLG, Gerald Klix)."

	^ Smalltalk platformName caseOf: {
		[ 'unix' ] -> [ '/home/bear/gitwork/Cuis-Smalltalk' ].
		[ 'Win32' ] -> [ 'h:\gitwork\Cuis-Smalltalk' ] }! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 4/26/2021 12:04:57'!
environmentRootDirectoryDefault
	"Answer the default value of environmentRootDirectory.
	
	Note that this is convenient for me (KLG, Gerald Klix)."

	^ Smalltalk platformName caseOf: {
		[ 'unix' ] -> [ '/home/bear/hgwork/Public-HaverOnCuis/environments/coreChanges' ].
		[ 'Win32' ] -> [ 'h:\hgwork\Public-HaverOnCuis\environments\coreChanges' ] }! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 5/29/2022 15:11:07'!
explicitUpdateNumberDefault
	"Answer the default for the explicit update number.
	
	nil is a valid answer resulting in the use of this image's update number."

	^ 5171! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 4/26/2021 12:06:24'!
haverRootDirectoryDefault
	"Answer the default value of haverRootDirectory
	
	Note that this is convenient for me (KLG, Gerald Klix)"

	^ Smalltalk platformName caseOf: {
		[ 'unix' ] -> [ '/home/bear/hgwork/Public-HaverOnCuis' ].
		[ 'Win32' ] -> [ 'h:\hgwork\Public-HaverOnCuis' ] }! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'defaults' stamp: 'KLG 4/26/2021 12:09:18'!
thirdPartyDirectoriesDefault
	"Answer the default value of thirdPartyDirectories.
	
	Note that this is convenient for me (KLG, Gerald Klix)"

	^ Smalltalk platformName caseOf: {
		[ 'unix' ] -> [
			#('/home/bear/gitwork/Cuis-Smalltalk-RegEx'
			'/home/bear/gitwork/Cuis-Smalltalk-X-ray') ].
		[ 'Win32' ] -> [
			#('h:\gitwork\Cuis-Smalltalk-RegEx'
			'h:\gitwork\Cuis-Smalltalk-X-ray') ] }! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/11/2021 15:04:56'!
cuisRootDirectory
	"Answer the value of cuisRootDirectory"

	^ cuisRootDirectory ifNil: [ self cuisRootDirectoryDefault ]! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/11/2021 14:59:53'!
cuisRootDirectory: anObject
	"Set the value of cuisRootDirectory"

	cuisRootDirectory _ anObject! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/11/2021 15:10:02'!
environmentRootDirectory
	"Answer the value of environmentRootDirectory.
	
	This value is used to locate the environment support change set."

	^ environmentRootDirectory ifNil: [ self environmentRootDirectoryDefault ]! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/11/2021 15:07:37'!
environmentRootDirectory: anObject
	"Set the value of environmentRootDirectory"

	environmentRootDirectory _ anObject! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/15/2021 14:40:22'!
explicitUpdateNumber
	"Answer the value of explicitUpdateNumber"

	^ explicitUpdateNumber ifNil: [ self explicitUpdateNumberDefault ]! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/15/2021 14:39:29'!
explicitUpdateNumber: anObject
	"Set the value of explicitUpdateNumber"

	explicitUpdateNumber _ anObject! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/11/2021 15:10:20'!
haverRootDirectory
	"Answer the value of haverRootDirectory"

	^ haverRootDirectory ifNil: [ self haverRootDirectoryDefault ]! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/11/2021 14:59:53'!
haverRootDirectory: anObject
	"Set the value of haverRootDirectory"

	haverRootDirectory _ anObject! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/12/2021 22:51:46'!
thirdPartyDirectories
	"Answer all the third party directories."
	
	^ thirdPartyDirectories ifNil: [ self thirdPartyDirectoriesDefault ]! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'accessing' stamp: 'KLG 4/12/2021 23:04:00'!
thirdPartyDirectories: anObject
	"Set the value of thirdPartyDirectories"

	thirdPartyDirectories _ anObject! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'versioning' stamp: 'KLG 5/29/2022 15:04:19'!
versionNumber
	"Answer the version number."

	^ 4.2! !

!(Modules>>#Distributor>>#Configuration) methodsFor: 'versioning' stamp: 'KLG 4/28/2021 13:56:41'!
versionSuffix
	"Answer the version suffix."

	^ 'alpha'! !

!(Modules>>#Distributor>>#Configuration) class methodsFor: 'accessing' stamp: 'KLG 4/11/2021 15:13:09'!
current
	"Answer the current configuration.
	
	For the time being we only support the default configuration"

	^ self default! !

!(Modules>>#Distributor>>#Configuration) class methodsFor: 'accessing' stamp: 'KLG 4/11/2021 14:59:28'!
default
	"Answer the value of default"

	^ default ifNil: [ default _ self new ]! !
