'From Haver 6.0 [latest update: #5170] on 17 May 2022 at 9:56:45 pm'!
'Description I contain old implementations of button morphs.'!
!provides: 'LegacyButtons' 1 3!
SystemOrganization addCategory: 'LegacyButtons'!


!moduleCreation: LegacyButtons!
Modules newEnvironment: #LegacyButtons!

!interfacesOf: LegacyButtons!
Modules environment: #LegacyButtons ::
	interface: #API exporting: #(#PluggableButtonMorph ) ::
	interface: #SPI aliasFor: #API ::
	interface: #UTI aliasFor: #API!


!classDefinition: (Modules>>#LegacyButtons>>#PluggableButtonMorph) category: 'LegacyButtons'!
PluggableMorph subclass: #PluggableButtonMorph
	instanceVariableNames: 'label font icon getStateSelector actionSelector isPressed mouseIsOver magnifiedIcon actWhen roundButtonStyle iconName secondaryActionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LegacyButtons'
	inModule: #LegacyButtons!
!classDefinition: (Modules>>#LegacyButtons>>#PluggableButtonMorph) class category: 'LegacyButtons'!
(Modules>>#LegacyButtons>>#PluggableButtonMorph) class
	instanceVariableNames: ''!


!(Modules>>#LegacyButtons>>#PluggableButtonMorph) commentStamp: '<historical>' prior: 0!
A PluggableButtonMorph is a combination of an indicator for a boolean value stored in its model and an action button. The action of a button is often, but not always, to toggle the boolean value that it shows. Its pluggable selectors are:

		getStateSelector		fetch a boolean value from the model
		actionSelector			invoke this button's action on the model
		getLabelSelector		fetch this button's lable from the model
		getMenuSelector		fetch a pop-up menu for this button from the model

Any of the above selectors can be nil, meaning that the model does not supply behavior for the given action, and the default behavior should be used. For example, if getStateSelector is nil, then this button shows the state of a read-only boolean that is always false.

The model informs its view(s) of changes by sending #changed: to itself with getStateSelector as a parameter. The view tells the model when the button is pressed by sending actionSelector.

If the actionSelector takes one or more arguments, then the following are relevant:
		arguments			A list of arguments to provide when the actionSelector is called.
		argumentsProvider	The object that is sent the argumentSelector to obtain arguments, if dynamic
		argumentsSelector	The message sent to the argumentProvider to obtain the arguments.!

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 1/4/2012 18:24'!
actWhen: condition
	"Accepts symbols:  #buttonDown, #buttonStillDown or #buttonUp (default)"
	actWhen _ condition! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing'!
action: aSymbol 
	"Set actionSelector to be the action defined by aSymbol."

	actionSelector _ aSymbol.
! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'sw 12/28/2000 16:17'!
actionSelector
	"Answer the receiver's actionSelector"

	^ actionSelector! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 12/30/2011 19:41'!
adoptWidgetsColor: paneColor
	super adoptWidgetsColor: paneColor.
	self color: (Theme current buttonColorFrom: paneColor)! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 7/19/2021 17:06:53'!
icon: aForm
	icon _ aForm.
	magnifiedIcon _ nil! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 7/19/2021 10:42:16'!
iconDrawSelector
	"Must be understood by drawing canvas."

	^iconName! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 7/19/2021 10:41:45'!
iconDrawSelector: aSymbol
	"aSymbol must be understood by drawing canvas."

	iconName _ aSymbol! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'KenD 9/17/2021 16:05:19'!
iconName
	
	^ self valueOfProperty: #iconName! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 7/19/2021 17:30:25'!
iconName: name
	self setProperty: #iconName toValue: name! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 7/18/2021 19:23:08'!
label
	^label! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 5/6/2009 08:18'!
label: aString
	"Label this button with the given string."

	self label: aString font: nil! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 4/24/2019 21:03:12'!
label: aStringOrNil font: aFontOrNil
	"Label this button with the given string."
	label _ aStringOrNil.
	font _ aFontOrNil.
	(self fontToUse notNil and: [ label notNil ])
		ifTrue: [ "Add a bit of padding"
			extent := (self fontToUse widthOfString: label) + 10 @ (self fontToUse lineSpacing + 10) ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 3/1/2011 14:07'!
performAction
	"Inform the model that this button has been pressed. "

	actionSelector ifNotNil: [
		model perform: actionSelector ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'accessing' stamp: 'jmv 5/24/2011 09:23'!
roundButtonStyle: aBooleanOrNil
	roundButtonStyle _ aBooleanOrNil! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'drawing' stamp: 'pb 3/17/2020 13:42:18'!
draw3DLookOn: aCanvas

	| borderStyleSymbol c |
	borderStyleSymbol _ self isPressed ifFalse: [ #raised ] ifTrue: [ #inset ].
	c _ color.
	self mouseIsOver ifTrue: [ c _ c  lighter ].
	aCanvas
		fillRectangle: self morphLocalBounds
		color: c
		borderWidth: borderWidth
		borderStyleSymbol: borderStyleSymbol
		baseColorForBorder: c.

	self drawRegularLabelOn: aCanvas! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'drawing' stamp: 'jmv 4/24/2019 20:58:58'!
drawEmbossedLabelOn: aCanvas

	| availableW center colorForLabel f l labelMargin targetSize w x y |
	label ifNotNil: [
		colorForLabel _ Theme current buttonLabel.
		self isPressed
			ifFalse: [
				self mouseIsOver
					ifFalse: [ colorForLabel _ colorForLabel adjustSaturation: -0.10 brightness: 0.10 ]]
			ifTrue: [ colorForLabel _ colorForLabel adjustSaturation: 0.0 brightness: -0.07 ].
		f _ self fontToUse.
		center _ extent // 2.
		labelMargin _ 3.
		w _ f widthOfString: label.
		availableW _ extent x - labelMargin - labelMargin.
		availableW >= w
			ifTrue: [
				l _ label ]
			ifFalse: [
				x _ labelMargin.
				targetSize _ label size * availableW // w.
				l _ label squeezedTo: targetSize.
				(f widthOfString: l) > availableW ifTrue: [
					targetSize _ targetSize - 1.
					l _ label squeezedTo: targetSize ]].
		
		w _ f widthOfString: l.
		x _ center x - (w // 2).
		y _ center y - (f lineSpacing // 2).
		aCanvas
			drawString: l
			at: x@y
			font: f
			color: colorForLabel
			embossed: true ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'drawing' stamp: 'jmv 7/20/2021 22:52:40'!
drawOn: aCanvas

	self isRoundButton
		ifTrue: [
			aCanvas drawButtonIconFromCurrentMorph ifFalse: [
				self drawRoundLookOn: aCanvas ]]
		ifFalse: [
			self draw3DLookOn: aCanvas.
			aCanvas drawButtonIconFromCurrentMorph ].! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'drawing' stamp: 'jmv 4/24/2019 20:59:05'!
drawRegularLabelOn: aCanvas

	| w f center x y  availableW l labelMargin |

	f _ self fontToUse.
	center _ extent // 2.

	label ifNotNil: [
		labelMargin _ 4.
		w _ f widthOfString: label.
		availableW _ extent x - labelMargin - labelMargin - 1.
		availableW >= w
			ifTrue: [
				x _ center x - (w // 2).
				l _ label ]
			ifFalse: [
				x _ labelMargin.
				l _ label squeezedTo: (label size * availableW / w) rounded ].
		y _ center y - (f lineSpacing // 2).
		self isPressed ifTrue: [
			x _ x + 1.
			y _ y + 1 ].
		aCanvas
			drawString: l
			at: x@y
			font: f
			color: Theme current buttonLabel ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'drawing' stamp: 'jmv 10/18/2021 11:58:24'!
drawRoundLookOn: aCanvas
	| r colorForButton rect |
	colorForButton _ self isPressed
		ifFalse: [
			self mouseIsOver
				ifTrue: [	 Color h: color hue s: color saturation * 1.3 v: color brightness * 0.9 ]
				ifFalse: [ color ]]
		ifTrue: [ color adjustSaturation: 0.1 brightness: -0.1 ].

	colorForButton ifNotNil: [
		r _ Theme current roundedButtonRadius.
		rect _ self morphLocalBounds insetBy: `3@3`.
		r _ r min: (rect width min: rect height) * 0.5.
		aCanvas roundRect: rect color: colorForButton radius: r ].

	Theme current embossedButtonLabels
		ifTrue: [ self drawEmbossedLabelOn: aCanvas ]
		ifFalse: [ self drawRegularLabelOn: aCanvas ].! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'drawing' stamp: 'KLG 5/17/2022 21:37:49'!
fontToUse
	| fontToUse |
	fontToUse _ font ifNil: [ PreferenceNG at: #standardButtonFont ].
	"
	Could add emphasis...
	^(emphasis isNil or: [emphasis = 0]) 
		ifTrue: [fontToUse]
		ifFalse: [fontToUse emphasized: emphasis]
	"
	^fontToUse! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'events' stamp: 'jmv 1/14/2013 22:47'!
mouseButton1Down: aMouseButtonEvent localPosition: localEventPosition

	isPressed _ true.
	self redrawNeeded.
	(actWhen == #buttonDown or: [ actWhen == #buttonStillDown ])
		ifTrue: [
			self performAction ]
		ifFalse: [
			"Don't make multi-click slower if we act on button down, just do multiple actions"
			aMouseButtonEvent hand
				waitForClicksOrDragOrSimulatedMouseButton2: self
				event: aMouseButtonEvent
				clkSel: nil
				clkNHalf: nil
				dblClkSel: #doubleClick:localPosition:
				dblClkNHalfSel: nil
				tripleClkSel: nil ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'events' stamp: 'jmv 10/28/2021 12:02:45'!
mouseButton1Up: aMouseButtonEvent localPosition: localEventPosition

	isPressed _ false.
	mouseIsOver _ false.
	(actWhen == #buttonUp and: [ 
		self includesPixel: aMouseButtonEvent eventPosition ])
		ifTrue: [ self performAction ].
	self redrawNeeded! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'events' stamp: 'jmv 5/24/2020 21:38:17'!
mouseButton2Up: aMouseButtonEvent localPosition: localEventPosition

	secondaryActionSelector ifNotNil: [
		model perform: secondaryActionSelector ].
	self redrawNeeded! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'events' stamp: 'jmv 8/20/2012 18:05'!
mouseEnter: event
	"The mouse entered the receiver"
	mouseIsOver _ true.
	self redrawNeeded.
	^super mouseEnter: event! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'events' stamp: 'jmv 8/20/2012 18:05'!
mouseLeave: event
	"The mouse has left the area of the receiver"
	mouseIsOver _ false.
	self redrawNeeded.
	^super mouseLeave: event! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'events' stamp: 'jmv 1/30/2014 00:25'!
mouseStillDown
	"Acting when down (instead of waiting until releasing the button)
	also means that the button actin is repeated if the button is kept pressed.
	See #handlesMouseStillDown:"
	self performAction! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'event handling testing' stamp: 'jmv 8/20/2012 18:57'!
handlesMouseDown: aMouseButtonEvent
	"Do I want to receive mouseDown events (mouseDown:, mouseMove:, mouseUp:)?"
	^true! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'event handling testing' stamp: 'jmv 8/20/2012 18:57'!
handlesMouseOver: evt
	"Do I want to receive mouseEnter: and mouseLeave: when the button is up and the hand is empty?" 
	^true! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'event handling testing' stamp: 'jmv 1/30/2014 00:27'!
handlesMouseStillDown: evt
	"Return true if the receiver wants to get repeated #mouseStillDown messages between #mouseDown: and #mouseUp"
	"Acting when down (instead of waiting until releasing the button)
	also means that the button action is repeated if the button is kept pressed"
	^actWhen == #buttonStillDown! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'event handling' stamp: 'jmv 1/30/2014 00:27'!
mouseStillDownStepRate
	"At what rate do I want to receive #mouseStillDown notifications?"
	^40! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'initialization' stamp: 'len 3/29/2021 10:36:35'!
defaultBorderWidth
	^ Theme current buttonBorderWidth! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'initialization' stamp: 'jmv 3/12/2018 15:54:57'!
initialize
	"initialize the state of the receiver"
	super initialize.

	roundButtonStyle _ nil.	"nil: honor Theme. true: draw as round button. false: draw as classic 3d border square button"
	model _ nil.
	getStateSelector _ nil.
	actionSelector _ nil.
	isPressed _ false.
	mouseIsOver _ false.
	actWhen _ #buttonUp.
	extent _  `20 @ 15`! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'initialization' stamp: 'jmv 6/2/2011 15:06'!
model: anObject
	"Set my model and make me me a dependent of the given object."

	model ifNotNil: [model removeDependent: self].
	getStateSelector ifNotNil: [
		anObject ifNotNil: [anObject addDependent: self]].
	model _ anObject! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'initialization' stamp: 'jmv 6/2/2011 15:07'!
model: anObject stateGetter: getStateSel action: actionSel label: aString

	getStateSelector _ getStateSel.
	actionSelector _ actionSel.
	self model: anObject.
	self label: aString! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'initialization' stamp: 'jmv 5/24/2020 21:36:57'!
secondaryActionSelector: actionSel

	secondaryActionSelector _ actionSel.! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'updating' stamp: 'jmv 9/25/2014 15:10'!
update: aSymbol

	super update: aSymbol.
	aSymbol = getStateSelector ifTrue: [
		self redrawNeeded ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'private' stamp: 'jmv 3/2/2010 16:21'!
getModelState
	"Answer the result of sending the receiver's model the getStateSelector message."

	^ getStateSelector 
		ifNil: [false]
		ifNotNil: [model perform: getStateSelector]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'private' stamp: 'jmv 7/19/2021 17:06:35'!
magnifiedIcon
	| factor magnifiedExtent w h |

	icon ifNil: [ ^nil ].
	magnifiedIcon ifNil: [
		magnifiedIcon _ icon.
		w _ icon width.
		h _ icon height.
		w*h = 0 ifFalse: [
			factor _ 1.0 * extent x / w min: 1.0 * extent y / h.
			factor = 1.0 ifFalse: [
				magnifiedExtent _ (icon extent * factor) rounded.
				magnifiedIcon _ icon magnifyTo: magnifiedExtent ]]].
	^magnifiedIcon! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'testing' stamp: 'jmv 9/2/2013 09:33'!
is: aSymbol
	^ aSymbol == #PluggableButtonMorph or: [ super is: aSymbol ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'testing' stamp: 'jmv 1/2/2012 16:43'!
isPressed
	^isPressed | self getModelState! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'testing' stamp: 'jmv 5/24/2011 09:24'!
isRoundButton
	^roundButtonStyle ifNil: [ Theme current roundButtons ]! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'testing' stamp: 'jmv 1/2/2012 16:43'!
mouseIsOver
	^mouseIsOver! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'geometry' stamp: 'jmv 11/15/2020 11:34:23'!
extentChanged: oldExtent
	"Our extent changed. Must layout submorphs again."

	super extentChanged: oldExtent.
	magnifiedIcon _ nil! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: 'geometry' stamp: 'KLG 5/17/2022 21:38:18'!
minimumExtent

	| e |
	e _ PreferenceNG at:  #windowTitleFont :: pointSize + 2.
	^e @ e! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) class methodsFor: 'example' stamp: 'jmv 3/12/2018 17:59:10'!
example
	"
	PluggableButtonMorph example openInWorld
	"

	| s1 s2 s3 b1 b2 b3 row |
	s1 _ Switch new.
	s2 _ Switch new turnOn.
	s3 _ Switch new.
	s2 onAction: [s3 turnOff].
	s3 onAction: [s2 turnOff].
	b1 _ (PluggableButtonMorph model: s1 stateGetter: #isOn action: #switch) label: 'S1'.
	b2 _ (PluggableButtonMorph model: s2 stateGetter: #isOn action: #turnOn) label: 'S2'.
	b3 _ (PluggableButtonMorph model: s3 stateGetter: #isOn action: #turnOn) label: 'S3'.
	b1 color: `Color lightRed`.
	b2 color: `Color lightRed`.
	b3 color: `Color lightRed`.
	row _ LayoutMorph newRow
		addMorphs: (Array with: b1 with: b2 with: b3);
		morphExtent: `120@35`.
	^ row! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) class methodsFor: 'instance creation' stamp: 'jmv 3/1/2011 15:00'!
model: anObject action: actionSel

	^ self new
		model: anObject
		stateGetter: nil
		action: actionSel
		label: nil
! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) class methodsFor: 'instance creation' stamp: 'jmv 3/16/2012 17:48'!
model: anObject action: actionSel label: aString

	^ self new
		model: anObject
		stateGetter: nil
		action: actionSel
		label: aString! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) class methodsFor: 'instance creation' stamp: 'jmv 3/1/2011 11:11'!
model: anObject stateGetter: getStateSel action: actionSel

	^ self new
		model: anObject
		stateGetter: getStateSel
		action: actionSel
		label: nil
! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) class methodsFor: 'instance creation' stamp: 'jmv 3/1/2011 11:10'!
model: anObject stateGetter: getStateSel action: actionSel label: aString
	^ self new
		model: anObject
		stateGetter: getStateSel
		action: actionSel
		label: aString! !
